from typing import Any, Callable, Type

from pypanther.base import Rule


def exclude(func: Callable[[Any], bool]):
    """
    Add a filter to exclude events from a rule. If `func` returns `True`, the `event` is excluded. Otherwise the rule is applied.

    Can be used as a decorator:
    ```
    # ignore events where event.foo == bar
    @exclude(lambda event: event["foo"] == "bar")
    class FilteredRule(TestRule):
        pass
    ```

    Can also be used standalone:
    ```
    # ignore events where event.foo == bar
    exclude(lambda event: event["foo"] == "bar")(FilteredRule)
    ```
    """

    def cls_wrapper(cls: Type[Rule]):
        _rule = cls.rule

        def wrapper(self, event):
            if func(event):
                return False
            return _rule(self, event)

        setattr(cls, "rule", wrapper)  # noqa: B010
        return cls

    return cls_wrapper


def include(func: Callable[[Any], bool]):
    """
    Add a filter to include events for a rule. If `func` returns `False`, the `event` is excluded. Otherwise the rule is applied.

    Can be used as a decorator:
    ```
    # require that event.foo == bar
    @include(lambda event: event["foo"] == "bar")
    class FilteredRule(TestRule):
        pass
    ```

    Can also be used standalone:
    ```
    # require that event.foo == bar
    include(lambda event: event["foo"] == "bar")(FilteredRule)
    ```
    """

    def cls_wrapper(cls: Type[Rule]):
        _rule = cls.rule

        def wrapper(self, event):
            if not func(event):
                return False
            return _rule(self, event)

        setattr(cls, "rule", wrapper)  # noqa: B010
        return cls

    return cls_wrapper



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import msft_graph_alert_context

microsoft_graph_passthrough_tests: list[RuleTest] = [
    RuleTest(
        name="Anonymous Login Event",
        expected_result=True,
        log={
            "azuretenantid": "12345-abcde-a1b2k3",
            "category": "AnonymousLogin",
            "createddatetime": "2022-08-04 14:31:48.438",
            "description": "Sign-in from an anonymous IP address (e.g. Tor browser, anonymizer VPNs)",
            "eventdatetime": "2022-08-04 14:31:48.438",
            "id": "abcd12345efghijk6789",
            "lastmodifieddatetime": "2022-08-04 14:34:56.229",
            "severity": "medium",
            "status": "newAlert",
            "title": "Anonymous IP address",
            "userstates": [
                {
                    "aadUserId": "011d5ede-0faa-4946-a25e-b2cd0c47a52c",
                    "accountName": "homer.simpson",
                    "domainName": "corporation.onmicrosoft.com",
                    "emailRole": "unknown",
                    "logonDateTime": "2022-08-04 14:31:48.438699700",
                    "logonIp": "185.220.103.6",
                    "logonLocation": "Brooklyn, New York, US",
                    "userPrincipalName": "homer.simpson@corporation.onmicrosoft.com",
                },
            ],
            "vendorinformation": {"provider": "IPC", "vendor": "Microsoft"},
        },
    ),
    RuleTest(
        name="Password Spray Event",
        expected_result=True,
        log={
            "azuretenantid": "abcdef-123456-ghijklmn",
            "category": "PasswordSpray",
            "createddatetime": "2022-08-17 09:28:04.767",
            "description": "Password spray attack detected",
            "eventdatetime": "2022-08-16 14:22:00.698",
            "id": "abcdefg-123456-hijklmno",
            "lastmodifieddatetime": "2022-08-17 14:30:21.979",
            "severity": "high",
            "status": "newAlert",
            "title": "Password Spray",
            "userstates": [
                {
                    "aadUserId": "abcdefg-123456-lmnop",
                    "accountName": "homer.simpson",
                    "domainName": "corporation.onmicrosoft.com",
                    "emailRole": "unknown",
                    "logonDateTime": "2022-08-16 14:22:00.698619500",
                    "logonIp": "109.70.100.21",
                    "logonLocation": "San Francisco, CA",
                    "userPrincipalName": "homer.simpson@corporation.onmicrosoft.com",
                },
            ],
            "vendorinformation": {"provider": "IPC", "vendor": "Microsoft"},
        },
    ),
    RuleTest(
        name="Resolved Event",
        expected_result=False,
        log={
            "azuretenantid": "abcdefg-12345",
            "category": "AnonymousLogin",
            "createddatetime": "2022-09-12 19:54:13.725",
            "description": "Sign-in from an anonymous IP address (e.g. Tor browser, anonymizer VPNs)",
            "eventdatetime": "2022-09-12 19:54:13.725",
            "id": "abcdefg12345hijklmnop",
            "lastmodifieddatetime": "2022-09-12 19:56:57.833",
            "severity": "high",
            "status": "resolved",
            "title": "Anonymous IP address",
            "userstates": [
                {
                    "aadUserId": "abcdefg1234",
                    "accountName": "homer.simpson",
                    "domainName": "corporation.onmicrosoft.com",
                    "emailRole": "unknown",
                    "logonDateTime": "2022-09-12 19:54:13.725589800",
                    "logonIp": "109.70.100.35",
                    "logonLocation": "San Francisco, CA",
                    "userPrincipalName": "homer.simpson@corporation.onmicrosoft.com",
                },
            ],
            "vendorinformation": {"provider": "IPC", "vendor": "Microsoft"},
        },
    ),
]


@panther_managed
class MicrosoftGraphPassthrough(Rule):
    default_description = "The Microsoft Graph security API federates queries to all onboarded security providers, including Azure AD Identity Protection, Microsoft 365, Microsoft Defender (Cloud, Endpoint, Identity) and Microsoft Sentinel"
    default_reference = "https://learn.microsoft.com/en-us/graph/api/resources/security-api-overview"
    display_name = "Microsoft Graph Passthrough"
    default_severity = Severity.MEDIUM
    log_types = [LogType.MICROSOFT_GRAPH_SECURITY_ALERT]
    id = "Microsoft.Graph.Passthrough-prototype"
    tests = microsoft_graph_passthrough_tests

    def rule(self, event):
        return event.get("status") == "newAlert"

    def title(self, event):
        return f"Microsoft Graph Alert ({event.get('title')})"

    def dedup(self, event):
        return event.get("id")

    def severity(self, event):
        if event.get("severity", "").lower() == "informational":
            return "INFO"
        return event.get("severity")

    def alert_context(self, event):
        return msft_graph_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import okta_alert_context

okta_user_account_locked_tests: list[RuleTest] = [
    RuleTest(
        name="Account Lock Event",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "abcd-1234"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Atlanta",
                    "country": "United States",
                    "geolocation": {"lat": 33, "lon": -80},
                    "postalCode": "30318",
                    "state": "Georgia",
                },
                "ipAddress": "1.2.3.4",
                "userAgent": {"browser": "CHROME", "os": "Mac OS X", "rawUserAgent": "Chrome"},
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "deviceFingerprint": "abc1234",
                    "dtHash": "000abc",
                    "requestId": "abc1111",
                    "requestUri": "/idp/idx/identify",
                    "threatSuspected": "false",
                    "url": "/idp/idx/identify?",
                },
            },
            "displaymessage": "Account Locked from New Devices - Max sign-in attempts exceeded.",
            "eventtype": "user.account.lock",
            "legacyeventtype": "core.user_auth.account_locked",
            "outcome": {"reason": "LOCKED_OUT", "result": "FAILURE"},
            "published": "2022-11-22 18:48:49.177",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Atlanta",
                            "country": "United States",
                            "geolocation": {"lat": 33, "lon": -80},
                            "postalCode": "30318",
                            "state": "Georgia",
                        },
                        "ip": "1.2.3.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 7018,
                "asOrg": "at&t corp.",
                "domain": "sbcglobal.net",
                "isProxy": False,
                "isp": "att services inc",
            },
            "severity": "DEBUG",
            "transaction": {"detail": {}, "id": "12345aaa", "type": "WEB"},
            "uuid": "aa-bb-cc-11",
            "version": "0",
        },
    ),
    RuleTest(
        name="Non Event",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpsons",
                "id": "00ABC123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "xyz1234"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 11.111, "lon": -70},
                    "postalCode": "1234",
                    "state": "California",
                },
                "ipAddress": "1.2.3.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "authnRequestId": "ABC123",
                    "deviceFingerprint": "009988771ABC",
                    "dtHash": "123abc1234",
                    "requestId": "abc-111-adf",
                    "requestUri": "/idp/idx/identify",
                    "threatSuspected": "false",
                    "url": "/idp/idx/identify?",
                },
            },
            "displaymessage": "Group Privilege granted",
            "eventtype": "group.privilege.grant",
            "legacyeventtype": "group.privilege.grant",
            "outcome": {"result": "FAILURE"},
            "published": "2022-12-13 00:58:19.811",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 11.111, "lon": -70},
                            "postalCode": "1234",
                            "state": "California",
                        },
                        "ip": "1.2.3.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 11351,
                "asOrg": "charter communications inc",
                "domain": "rr.com",
                "isProxy": False,
                "isp": "charter communications inc",
            },
            "severity": "WARN",
            "target": [{"alternateId": "App (123)", "displayName": "App (123)", "id": "12345", "type": "AppInstance"}],
            "transaction": {"detail": {}, "id": "aaa-bbb-123", "type": "WEB"},
            "uuid": "aa-11-22-33-44-bb",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaUserAccountLocked(Rule):
    default_description = "An Okta user has locked their account."
    display_name = "Okta User Account Locked"
    default_reference = "https://support.okta.com/help/s/article/How-to-Configure-the-Number-of-Failed-Login-Attempts-Before-User-Lockout?language=en_US"
    default_severity = Severity.LOW
    log_types = [LogType.OKTA_SYSTEM_LOG]
    id = "Okta.User.Account.Locked-prototype"
    tests = okta_user_account_locked_tests

    def rule(self, event):
        return event.get("eventtype") in ("user.account.lock", "user.account.lock.limit")

    def title(self, event):
        return f"Okta: [{event.get('actor', {}).get('alternateId', '<id-not-found>')}] [{event.get('displaymessage', 'account has been locked.')}]"

    def alert_context(self, event):
        return okta_alert_context(event)



import logging
import sys

from colorama import Fore, Style, init


class CustomFormatter(logging.Formatter):
    format_str = "[%(levelname)s][%(name)s]: %(message)s"

    FORMATS = {
        logging.DEBUG: format_str,
        logging.INFO: format_str,
        logging.WARNING: Fore.YELLOW + format_str + Style.RESET_ALL,
        logging.ERROR: Fore.RED + format_str + Style.RESET_ALL,
        logging.CRITICAL: Fore.RED + format_str + Style.RESET_ALL,
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


def setup_logging():
    # setup logger and print version info as necessary
    if sys.stderr.isatty():
        init()
        logger = logging.getLogger()
        logger.setLevel(logging.INFO)
        handler = logging.StreamHandler()
        handler.setLevel(logging.INFO)
        handler.setFormatter(CustomFormatter())
        logger.addHandler(handler)
    else:
        logging.basicConfig(
            format="[%(levelname)s][%(name)s]: %(message)s",
            level=logging.INFO,
        )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

okta_ss_oto_aws_tests: list[RuleTest] = [
    RuleTest(
        name="AWS SSO via Okta",
        expected_result=True,
        log={
            "displayMessage": "User single sign on to app",
            "eventType": "user.authentication.sso",
            "legacyEventType": "app.auth.sso",
            "outcome": {"result": "SUCCESS"},
            "securityContext": {},
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "AWS Production",
                    "detailEntry": {"signOnModeType": "SAML_2_0"},
                    "displayName": "AWS IAM Identity Center",
                    "id": "0oaua5ldoougycQAO696",
                    "type": "AppInstance",
                },
                {"alternateId": "aardvark", "displayName": "aardvark", "id": "0ua8aardvarkD697", "type": "AppUser"},
            ],
            "transaction": {"detail": {}, "id": "1a3852fc0d172ecdad0e2447e47fbc98", "type": "WEB"},
            "uuid": "35cae732-21bd-11ef-a011-dd05aa53a11a",
            "version": "0",
        },
    ),
    RuleTest(
        name="AWS SSO via Okta without app name",
        expected_result=False,
        log={
            "displayMessage": "User single sign on to app",
            "eventType": "user.authentication.sso",
            "legacyEventType": "app.auth.sso",
            "outcome": {"result": "SUCCESS"},
            "securityContext": {},
            "severity": "INFO",
            "target": [{"alternateId": "aardvark", "id": "0ua8aardvarkD697", "type": "AppUser"}],
            "transaction": {"detail": {}, "id": "1a3852fc0d172ecdad0e2447e47fbc98", "type": "WEB"},
            "uuid": "35cae732-21bd-11ef-a011-dd05aa53a11a",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaSSOtoAWS(Rule):
    id = "Okta.SSO.to.AWS-prototype"
    display_name = "SIGNAL - Okta SSO to AWS"
    create_alert = False
    log_types = [LogType.OKTA_SYSTEM_LOG]
    default_severity = Severity.INFO
    tests = okta_ss_oto_aws_tests

    def rule(self, event):
        return all(
            [
                event.get("eventType") == "user.authentication.sso",
                event.deep_get("outcome", "result") == "SUCCESS",
                "AWS IAM Identity Center" in event.deep_walk("target", "displayName", default=[]),
            ],
        )



import time

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_notion_helpers import notion_alert_context
from pypanther.helpers.panther_oss_helpers import get_string_set, put_string_set

notion_account_changed_after_login_tests: list[RuleTest] = [
    RuleTest(
        name="Login event",
        expected_result=True,
        mocks=[RuleMock(object_name="put_string_set", return_value=True)],
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "user.login",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
    RuleTest(
        name="Email Changed Shortly After Login",
        expected_result=True,
        mocks=[
            RuleMock(object_name="get_string_set", return_value='[\n  "2023-06-12 21:40:28.690000000"\n]'),
            RuleMock(object_name="put_string_set", return_value=""),
        ],
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "user.settings.login_method.email_updated",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {
                "ipinfo_location": {
                    "event.ip_address": {
                        "city": "Barad-Dur",
                        "lat": "0.00000",
                        "lng": "0.00000",
                        "country": "Mordor",
                        "postal_code": "55555",
                        "region": "Mount Doom",
                        "region_code": "MD",
                        "timezone": "Middle Earth/Mordor",
                    },
                },
            },
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
    RuleTest(
        name="Password Changed Shortly After Login",
        expected_result=True,
        mocks=[
            RuleMock(object_name="get_string_set", return_value='[\n  "2023-06-12 21:40:28.690000000"\n]'),
            RuleMock(object_name="put_string_set", return_value=""),
        ],
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "user.settings.login_method.password_updated",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {
                "ipinfo_location": {
                    "event.ip_address": {
                        "city": "Barad-Dur",
                        "lat": "0.00000",
                        "lng": "0.00000",
                        "country": "Mordor",
                        "postal_code": "55555",
                        "region": "Mount Doom",
                        "region_code": "MD",
                        "timezone": "Middle Earth/Mordor",
                    },
                },
            },
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
    RuleTest(
        name="Password Added Shortly After Login",
        expected_result=True,
        mocks=[
            RuleMock(object_name="get_string_set", return_value='[\n  "2023-06-12 21:40:28.690000000"\n]'),
            RuleMock(object_name="put_string_set", return_value=""),
        ],
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "user.settings.login_method.password_added",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {
                "ipinfo_location": {
                    "event.ip_address": {
                        "city": "Barad-Dur",
                        "lat": "0.00000",
                        "lng": "0.00000",
                        "country": "Mordor",
                        "postal_code": "55555",
                        "region": "Mount Doom",
                        "region_code": "MD",
                        "timezone": "Middle Earth/Mordor",
                    },
                },
            },
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
    RuleTest(
        name="Password Removed Shortly After Login",
        expected_result=True,
        mocks=[
            RuleMock(object_name="get_string_set", return_value='[\n  "2023-06-12 21:40:28.690000000"\n]'),
            RuleMock(object_name="put_string_set", return_value=""),
        ],
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "user.settings.login_method.password_removed",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {
                "ipinfo_location": {
                    "event.ip_address": {
                        "city": "Barad-Dur",
                        "lat": "0.00000",
                        "lng": "0.00000",
                        "country": "Mordor",
                        "postal_code": "55555",
                        "region": "Mount Doom",
                        "region_code": "MD",
                        "timezone": "Middle Earth/Mordor",
                    },
                },
            },
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
    RuleTest(
        name="Email Changed Not Shortly After Login",
        expected_result=False,
        mocks=[
            RuleMock(object_name="get_string_set", return_value=False),
            RuleMock(object_name="put_string_set", return_value=""),
        ],
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "user.settings.login_method.email_updated",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {
                "ipinfo_location": {
                    "event.ip_address": {
                        "city": "Barad-Dur",
                        "lat": "0.00000",
                        "lng": "0.00000",
                        "country": "Mordor",
                        "postal_code": "55555",
                        "region": "Mount Doom",
                        "region_code": "MD",
                        "timezone": "Middle Earth/Mordor",
                    },
                },
            },
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
    RuleTest(
        name="Unrelated event",
        expected_result=False,
        log={
            "event": {
                "actor": {
                    "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                    "object": "user",
                    "person": {"email": "aragorn.elessar@lotr.com"},
                    "type": "person",
                },
                "details": {"authType": "email"},
                "id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "ip_address": "192.168.100.100",
                "platform": "web",
                "timestamp": "2023-06-12 21:40:28.690000000",
                "type": "page.viewed",
                "workspace_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            },
            "p_enrichment": {
                "ipinfo_location": {
                    "event.ip_address": {
                        "city": "Barad-Dur",
                        "lat": "0.00000",
                        "lng": "0.00000",
                        "country": "Mordor",
                        "postal_code": "55555",
                        "region": "Mount Doom",
                        "region_code": "MD",
                        "timezone": "Middle Earth/Mordor",
                    },
                },
            },
            "p_event_time": "2023-06-12 21:40:28.690000000",
            "p_log_type": "Notion.AuditLogs",
            "p_parse_time": "2023-06-12 22:53:51.602223297",
            "p_row_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "p_schema_version": 0,
            "p_source_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "p_source_label": "Notion Logs",
        },
    ),
]


@panther_managed
class NotionAccountChangedAfterLogin(Rule):
    id = "Notion.AccountChangedAfterLogin-prototype"
    display_name = "Notion Account Changed Shortly After Login"
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Identity & Access Management", "Persistence"]
    default_severity = Severity.MEDIUM
    default_description = "A Notion User logged in then changed their account details."
    default_runbook = (
        "Possible account takeover. Follow up with the Notion User to determine if this email change is genuine."
    )
    default_reference = "https://www.notion.so/help/account-settings"
    tests = notion_account_changed_after_login_tests
    # Length of time in minutes. If a user logs in, then changes their email within this many
    # minutes, raise an alert.
    DEFAULT_EMAIL_CHANGE_WINDOW_MINUTES = 10
    # Prefix for cached key. This ensures we don't accidently tamper with cached data from other
    # detections.
    CACHE_PREFIX = "Notion.AccountChangedAfterLogin"
    LOGIN_TS = None  # Default Value

    def rule(self, event):
        # If this is neither a login, nor an email/password change event, then exit
        allowed_event_types = {
            "user.login",
            "user.settings.login_method.email_updated",
            "user.settings.login_method.password_updated",
            "user.settings.login_method.password_added",
            "user.settings.login_method.password_removed",
        }
        if event.deep_walk("event", "type") not in allowed_event_types:
            return False
        # Global Variable Stuff
        # Extract user info
        userid = event.deep_walk("event", "actor", "id")
        cache_key = f"{self.CACHE_PREFIX}-{userid}"
        # If this is a login event, record it
        if event.deep_walk("event", "type") == "user.login":
            # Returning this as a bool allows us to write a unit test to determine if we cache login
            #   events when we're supposed to.
            # We'll save this for the alert context later
            return bool(
                put_string_set(
                    cache_key,
                    [str(event.get("p_event_time"))],
                    time.time() + self.DEFAULT_EMAIL_CHANGE_WINDOW_MINUTES * 60,
                ),
            )
        # If we made it here, then this is an account change event.
        # We first check if the user recently logged in:
        if last_login := get_string_set(cache_key, force_ttl_check=True):
            self.LOGIN_TS = list(last_login)[0]  # Save the last login timestamp for the alert context
            return True
        # If they haven't logged in recently, then return false
        return False

    def title(self, event):
        user_email = event.deep_walk("event", "actor", "person", "email", default="UNKNOWN EMAIL")
        mins = self.DEFAULT_EMAIL_CHANGE_WINDOW_MINUTES
        action_taken = {
            "user.settings.login_method.email_updated": "changed their email",
            "user.settings.login_method.password_updated": "changed their password",
            "user.settings.login_method.password_added": "added a password to their account",
            "user.settings.login_method.password_removed": "removed the password from their account",
        }.get(event.deep_get("event", "type"), "altered their account info")
        return f"Notion User [{user_email}] {action_taken} within [{mins}] minutes of logging in."

    def alert_context(self, event):
        context = notion_alert_context(event)
        if self.LOGIN_TS:
            context["login_timestamp"] = self.LOGIN_TS
        return context



from pypanther import LogType, Rule, Severity, panther_managed
from pypanther.helpers.panther_notion_helpers import notion_alert_context


@panther_managed
class NotionLoginFromBlockedIP(Rule):
    id = "Notion.LoginFromBlockedIP-prototype"
    display_name = "Notion Login From Blocked IP"
    enabled = False
    log_types = [LogType.NOTION_AUDIT_LOGS]
    tags = ["Notion", "Network Security Monitoring", "Malicious Connections", "Configuration Required"]
    default_severity = Severity.MEDIUM
    default_description = "A user attempted to access Notion from a blocked IP address. Note: before deployinh, make sure to add Rule Filters checking if event.ip_address is in a certain CIDR range(s)."
    default_runbook = "Confirm with user if the login was legitimate. If so, determine why the IP is blocked."
    default_reference = "https://www.notion.so/help/allowlist-ip"

    def rule(self, event):
        # Users can specify inline-filters to permit rules based on IPs
        return event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>") == "user.login"

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        ip_addr = event.deep_get("event", "ip_address", default="<UNKNOWN IP>")
        return f"Notion User [{user}] attempted to login from a blocked IP: [{ip_addr}]."

    def alert_context(self, event):
        return notion_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk, okta_alert_context

okta_identity_provider_sign_in_tests: list[RuleTest] = [
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                },
            },
            "displaymessage": "Authentication of user via MFA",
            "eventtype": "user.authentication.auth_via_mfa",
            "legacyeventtype": "core.user.factor.attempt_fail",
            "outcome": {"reason": "INVALID_CREDENTIALS", "result": "FAILURE"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
    RuleTest(
        name="FastPass Phishing Block Event",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                },
            },
            "displaymessage": "Authentication of user via MFA",
            "eventtype": "user.authentication.auth_via_IDP",
            "legacyeventtype": "core.user.factor.attempt_fail",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaIdentityProviderSignIn(Rule):
    id = "Okta.Identity.Provider.SignIn-prototype"
    display_name = "Okta Identity Provider Sign-in"
    enabled = False
    log_types = [LogType.OKTA_SYSTEM_LOG]
    tags = ["Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0001:T1199", "TA0003:T1098"]}
    default_severity = Severity.HIGH
    default_description = 'A user has signed in using a 3rd party Identity Provider. Attackers have been observed configuring a second Identity Provider to act as an "impersonation app" to access applications within the compromised Org on behalf of other users. This second Identity Provider, also controlled by the attacker, would act as a “source” IdP in an inbound federation relationship (sometimes called “Org2Org”) with the target. From this “source” IdP, the threat actor manipulated the username parameter for targeted users in the second “source” Identity Provider to match a real user in the compromised “target” Identity Provider. This provided the ability to Single sign-on (SSO) into applications in the target IdP as the targeted user. Do not use this rule if your organization uses legitimate 3rd-party Identity Providers.\n'
    default_reference = "https://sec.okta.com/articles/2023/08/cross-tenant-impersonation-prevention-and-detection\n"
    dedup_period_minutes = 30
    tests = okta_identity_provider_sign_in_tests

    def rule(self, event):
        return event.get("eventType") == "user.authentication.auth_via_IDP"

    def title(self, event):
        target = deep_walk(event, "target", "displayName", default="displayName-not-found", return_val="first")
        return f"{deep_get(event, 'actor', 'displayName', default='<displayName-not-found>')} <{deep_get(event, 'actor', 'alternateId', default='alternateId-not-found')}> signed in via 3rd party Identity Provider to {target}"

    def alert_context(self, event):
        return okta_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_auth0_helpers import auth0_alert_context, is_auth0_config_event
from pypanther.helpers.panther_base_helpers import deep_get

auth0_post_login_action_flow_tests: list[RuleTest] = [
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "data": {
                "client_id": "XXX",
                "client_name": "",
                "date": "2023-05-16 17:28:11.165000000",
                "description": "Set the Multi-factor Authentication policies",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {
                                "jti": "XXX",
                                "scopes": ["create:actions", "update:triggers", "update:users"],
                            },
                            "strategy": "jwt",
                            "user": {
                                "email": "user.name@yourcompany.io",
                                "name": "User Name",
                                "user_id": "google-oauth2|XXX",
                            },
                        },
                        "body": [],
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "put",
                        "path": "/api/v2/guardian/policies",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {"body": [], "statusCode": 200},
                },
                "ip": "12.12.12.12",
                "log_id": "XXX",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "google-oauth2|XXX",
            },
            "log_id": "XXX",
            "p_source_label": "Org Tenant Label",
        },
    ),
    RuleTest(
        name="Action Added",
        expected_result=True,
        log={
            "data": {
                "client_id": "XXX",
                "client_name": "",
                "date": "2023-05-23 20:53:36.557000000",
                "description": "Update trigger bindings",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {"jti": "XXX"},
                            "strategy": "jwt",
                            "user": {
                                "email": "homer.simpson@yourcompany.com",
                                "name": "Homer Simpson",
                                "user_id": "google-oauth2|XXX",
                            },
                        },
                        "body": {
                            "bindings": [
                                {"display_name": "Password Rotation", "ref": {"type": "action_id", "value": "XXX"}},
                            ],
                        },
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "patch",
                        "path": "/api/v2/actions/triggers/post-login/bindings",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {
                        "body": {
                            "bindings": [
                                {
                                    "action": {
                                        "all_changes_deployed": False,
                                        "created_at": "2023-04-24T19:33:44.217168082Z",
                                        "current_version": {
                                            "created_at": "2023-05-11T17:26:53.569382908Z",
                                            "id": "c4c1d580-2f51-4d7b-afa5-ad4216f40ad3",
                                            "runtime": "node16",
                                            "status": "BUILT",
                                            "updated_at": "2023-05-11T17:26:53.724617041Z",
                                        },
                                        "deployed_version": {
                                            "code": "",
                                            "created_at": "2023-05-11T17:26:53.569382908Z",
                                            "dependencies": [],
                                            "deployed": True,
                                            "id": "c4c1d580-2f51-4d7b-afa5-ad4216f40ad3",
                                            "runtime": "node16",
                                            "secrets": [],
                                            "status": "built",
                                            "updated_at": "2023-05-11T17:26:53.724617041Z",
                                        },
                                        "id": "XXX",
                                        "installed_integration_id": "XXX",
                                        "integration": {
                                            "catalog_id": "password-rotation",
                                            "created_at": "2023-04-24T19:33:44.212805393Z",
                                            "current_release": {"id": "", "semver": {}},
                                            "description": "This Action allows you to configure the number of days a password is valid before it expires and the user must rotate it.\n",
                                            "feature_type": "action",
                                            "id": "64bee519-818f-4473-ab08-7c380f28da77",
                                            "logo": "https://cdn.auth0.com/marketplace/catalog/content/assets/creators/auth0/auth0-avatar.png",
                                            "name": "Password Rotation",
                                            "partner_id": "XXX",
                                            "short_description": "Enforce Users to Rotate Passwords",
                                            "terms_of_use_url": "https://cdn.auth0.com/website/legal/files/mktplace/auth0-integration.pdf",
                                            "updated_at": "2023-05-11T17:26:53.560940001Z",
                                            "url_slug": "auth-0-password-rotation",
                                        },
                                        "name": "Password Rotation v1",
                                        "supported_triggers": [
                                            {"id": "post-login", "status": "CURRENT", "version": "v3"},
                                        ],
                                        "updated_at": "2023-04-24T19:33:44.217168082Z",
                                    },
                                    "created_at": "2023-05-23T20:53:36.528608347Z",
                                    "display_name": "Password Rotation",
                                    "id": "a12b9e2a-ec0f-4060-b476-18547030088a",
                                    "trigger_id": "post-login",
                                    "updated_at": "2023-05-23T20:53:36.528608347Z",
                                },
                            ],
                        },
                        "statusCode": 200,
                    },
                },
                "ip": "12.12.12.12",
                "log_id": "XXX",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "google-oauth2|XXX",
            },
            "log_id": "XXX",
            "p_source_label": "Org Auth0 Tenant Label",
        },
    ),
    RuleTest(
        name="Action Removed",
        expected_result=True,
        log={
            "data": {
                "client_id": "XXX",
                "client_name": "",
                "date": "2023-05-23 21:35:54.491000000",
                "description": "Update trigger bindings",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {"jti": "e6343ec1d24a41e6bd43a6be748cac11"},
                            "strategy": "jwt",
                            "user": {
                                "email": "homer.simpson@yourcompany.com",
                                "name": "Homer Simpson",
                                "user_id": "google-oauth2|XXX",
                            },
                        },
                        "body": {"bindings": [{"ref": {"type": "binding_id", "value": "XXX"}}]},
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "patch",
                        "path": "/api/v2/actions/triggers/post-login/bindings",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {
                        "body": {
                            "bindings": [
                                {
                                    "action": {
                                        "all_changes_deployed": False,
                                        "created_at": "2022-05-26T20:45:09.128843683Z",
                                        "current_version": {
                                            "created_at": "2022-05-26T20:45:09.128849956Z",
                                            "id": "XXX",
                                            "runtime": "node12",
                                            "status": "BUILT",
                                            "updated_at": "2022-05-26T20:45:32.857727217Z",
                                        },
                                        "deployed_version": {
                                            "code": "",
                                            "created_at": "2022-05-26T20:45:09.128849956Z",
                                            "dependencies": [],
                                            "deployed": True,
                                            "id": "XXX",
                                            "runtime": "node12",
                                            "secrets": [],
                                            "status": "built",
                                            "updated_at": "2022-05-26T20:45:32.857727217Z",
                                        },
                                        "id": "XXX",
                                        "installed_integration_id": "XXX",
                                        "integration": {
                                            "catalog_id": "auth0-country-based-access",
                                            "created_at": "2022-05-26T20:45:09.064825654Z",
                                            "current_release": {"id": "", "semver": {}},
                                            "description": "This integration allows you to restrict access to your applications by country. You may choose to implement Country-based Access controls for various reasons, including to allow your applications to comply with unique restrictions based on where you do business. \n\nWith the Country-based Access integration, you can define any and all countries to restrict persons and entities from those countries logging into your applications. ",
                                            "feature_type": "action",
                                            "id": "XXX",
                                            "logo": "https://cdn.auth0.com/marketplace/catalog/content/assets/creators/auth0/auth0-avatar.png",
                                            "name": "Country-based Access",
                                            "partner_id": "XXX",
                                            "public_support_link": "https://support.auth0.com/",
                                            "short_description": "Restrict access to users by country",
                                            "terms_of_use_url": "https://cdn.auth0.com/website/legal/files/mktplace/auth0-integration.pdf",
                                            "updated_at": "2022-05-26T20:45:09.064825654Z",
                                            "url_slug": "country-based-access",
                                        },
                                        "name": "Country-based Access v2",
                                        "supported_triggers": [
                                            {"id": "post-login", "status": "CURRENT", "version": "v2"},
                                        ],
                                        "updated_at": "2022-05-26T20:45:09.128843683Z",
                                    },
                                    "created_at": "2023-05-23T21:35:47.983852439Z",
                                    "display_name": "Country-based Access",
                                    "id": "XXX",
                                    "trigger_id": "post-login",
                                    "updated_at": "2023-05-23T21:35:54.464801212Z",
                                },
                            ],
                        },
                        "statusCode": 200,
                    },
                },
                "ip": "12.12.12.12",
                "log_id": "XXX",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "google-oauth2|XXX",
            },
            "log_id": "XXX",
            "p_source_label": "Org Auth0 Tenant Label",
        },
    ),
    RuleTest(
        name="All Actions Removed",
        expected_result=True,
        log={
            "data": {
                "client_id": "XXX",
                "client_name": "",
                "date": "2023-05-23 21:28:56.071000000",
                "description": "Update trigger bindings",
                "details": {
                    "request": {
                        "auth": {
                            "credentials": {"jti": "XXX"},
                            "strategy": "jwt",
                            "user": {
                                "email": "homer.simpson@yourcompany.com",
                                "name": "Homer Simpson",
                                "user_id": "google-oauth2|XXX",
                            },
                        },
                        "body": {"bindings": []},
                        "channel": "https://manage.auth0.com/",
                        "ip": "12.12.12.12",
                        "method": "patch",
                        "path": "/api/v2/actions/triggers/post-login/bindings",
                        "query": {},
                        "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                    },
                    "response": {"body": {"bindings": []}, "statusCode": 200},
                },
                "ip": "12.12.12.12",
                "log_id": "XXX",
                "type": "sapi",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36",
                "user_id": "google-oauth2|XXX",
            },
            "log_id": "XXX",
            "p_source_label": "Org Auth0 Tenant Label",
        },
    ),
]


@panther_managed
class Auth0PostLoginActionFlow(Rule):
    default_description = "An Auth0 User updated a post login action flow for your organization's tenant."
    display_name = "Auth0 Post Login Action Flow Updated"
    default_runbook = "Assess if this was done by the user for a valid business reason. Be sure to replace any steps that were removed without authorization."
    default_reference = "https://auth0.com/docs/customize/actions/flows-and-triggers/login-flow/api-object"
    default_severity = Severity.MEDIUM
    log_types = [LogType.AUTH0_EVENTS]
    id = "Auth0.Post.Login.Action.Flow-prototype"
    tests = auth0_post_login_action_flow_tests

    def rule(self, event):
        data_description = deep_get(event, "data", "description", default="<NO_DATA_DESCRIPTION_FOUND>")
        request_path = deep_get(event, "data", "details", "request", "path", default="<NO_REQUEST_PATH_FOUND>")
        return all(
            [
                data_description == "Update trigger bindings",
                request_path == "/api/v2/actions/triggers/post-login/bindings",
                is_auth0_config_event(event),
            ],
        )

    def title(self, event):
        user = deep_get(event, "data", "details", "request", "auth", "user", "email", default="<NO_USER_FOUND>")
        p_source_label = deep_get(event, "p_source_label", default="<NO_P_SOURCE_LABEL_FOUND>")
        request_bindings = deep_get(event, "data", "details", "request", "body", "bindings", default=[])
        response_bindings = deep_get(event, "data", "details", "response", "body", "bindings", default=[])
        actions_added_list = []
        for binding in request_bindings:
            if "display_name" in binding:
                # check to see if actions were added to the flow
                actions_added_list.append(binding.get("display_name", ""))
        # otherwise, actions were removed from the action flow and we want
        # to grab what's still present in the flow
        actions_remaining_list = []
        for binding in response_bindings:
            if deep_get(binding, "display_name"):
                actions_remaining_list.append(deep_get(binding, "display_name", default="<NO_DISPLAYNAME>"))
        if actions_added_list:
            return f"Auth0 User [{user}] added action(s) [{actions_added_list}] to a post-login action flow for your organization’s tenant [{p_source_label}]."
        if actions_remaining_list:
            return f"Auth0 User [{user}] removed action(s) to a post-login action flow for your organization’s tenant [{p_source_label}], remaining actions include [{actions_remaining_list}]."
        # no actions remain in the flow
        return f"Auth0 User [{user}] removed all actions from a post-login action flow for your organization’s tenant [{p_source_label}]."

    def alert_context(self, event):
        return auth0_alert_context(event)



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

standard_mfa_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - Org MFA Disabled",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "org.disable_two_factor_requirement",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repository": "my-org/my-repo",
        },
    ),
    RuleTest(
        name="GitHub - Org MFA Enabled",
        expected_result=False,
        log={
            "actor": "cat",
            "action": "org.enable_two_factor_requirement",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repository": "my-org/my-repo",
        },
    ),
    RuleTest(
        name="Zendesk - Two-factor disabled",
        expected_result=True,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Two-Factor authentication for all admins and agents",
            "action": "create",
            "change_description": "Disabled",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Zendesk - Alternate Two-factor disabled",
        expected_result=True,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Require Two Factor",
            "action": "update",
            "change_description": "Disabled",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Zendesk - Two-factor Enabled",
        expected_result=False,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "source_id": 123,
            "source_type": "account_setting",
            "source_label": "Require Two Factor",
            "action": "create",
            "change_description": "Enabled",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Okta - User resets own MFA ( INFO severity detection for this in Okta rules )",
        expected_result=False,
        log={
            "eventtype": "user.mfa.factor.deactivate",
            "version": "0",
            "severity": "INFO",
            "displaymessage": "Reset factor for user",
            "actor": {
                "alternateId": "homer@springfield.gov",
                "displayName": "Homer Simpson",
                "id": "11111111111",
                "type": "User",
            },
            "client": {
                "device": "Computer",
                "ipAddress": "1.1.1.1",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.55 Safari/537.36",
                },
                "zone": "null",
            },
            "outcome": {"reason": "User reset FIDO_WEBAUTHN factor", "result": "SUCCESS"},
            "target": [
                {
                    "alternateId": "homer@springfield.gov",
                    "displayName": "Homer Simpson",
                    "id": "1111111",
                    "type": "User",
                },
            ],
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "1111111"},
            "p_log_type": "Okta.SystemLog",
        },
    ),
    RuleTest(
        name="Okta - User disables MFA",
        expected_result=True,
        log={
            "eventtype": "user.mfa.factor.deactivate",
            "version": "0",
            "severity": "INFO",
            "displaymessage": "Reset factor for user",
            "actor": {
                "alternateId": "homer@springfield.gov",
                "displayName": "Homer Simpson",
                "id": "11111111111",
                "type": "User",
            },
            "client": {
                "device": "Computer",
                "ipAddress": "1.1.1.1",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.55 Safari/537.36",
                },
                "zone": "null",
            },
            "outcome": {"result": "SUCCESS"},
            "target": [
                {
                    "alternateId": "homer@springfield.gov",
                    "displayName": "Homer Simpson",
                    "id": "1111111",
                    "type": "User",
                },
            ],
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "1111111"},
            "p_log_type": "Okta.SystemLog",
        },
    ),
    RuleTest(
        name="Okta - MFA Enabled",
        expected_result=False,
        log={
            "eventtype": "user.mfa.factor.update",
            "version": "0",
            "severity": "INFO",
            "displaymessage": "Update factor for user",
            "actor": {
                "alternateId": "homer@springfield.gov",
                "displayName": "Homer Simpson",
                "id": "11111111111",
                "type": "User",
            },
            "client": {
                "device": "Computer",
                "ipAddress": "1.1.1.1",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.55 Safari/537.36",
                },
                "zone": "null",
            },
            "outcome": {"reason": "User reset FIDO_WEBAUTHN factor", "result": "SUCCESS"},
            "target": [
                {
                    "alternateId": "homer@springfield.gov",
                    "displayName": "Homer Simpson",
                    "id": "1111111",
                    "type": "User",
                },
            ],
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "1111111"},
            "p_log_type": "Okta.SystemLog",
        },
    ),
]


@panther_managed
class StandardMFADisabled(Rule):
    id = "Standard.MFADisabled-prototype"
    display_name = "MFA Disabled"
    log_types = [LogType.ATLASSIAN_AUDIT, LogType.GITHUB_AUDIT, LogType.ZENDESK_AUDIT, LogType.OKTA_SYSTEM_LOG]
    tags = ["DataModel", "Defense Evasion:Modify Authentication Process"]
    reports = {"MITRE ATT&CK": ["TA0005:T1556"]}
    default_reference = "https://en.wikipedia.org/wiki/Multi-factor_authentication"
    default_severity = Severity.HIGH
    default_description = "Detects when Multi-Factor Authentication (MFA) is disabled"
    summary_attributes = ["p_any_ip_addresses"]
    tests = standard_mfa_disabled_tests

    def rule(self, event):
        return event.udm("event_type") == event_type.MFA_DISABLED

    def title(self, event):
        # use unified data model field in title
        return f"{event.get('p_log_type')}: User [{event.udm('actor_user')}] disabled MFA"



from pypanther.base import DataModel, DataModelMapping, LogType


class StandardAWSALB(DataModel):
    id: str = "Standard.AWS.ALB"
    display_name: str = "AWS Application Load Balancer"
    enabled: bool = True
    log_types: list[str] = [LogType.AWS_ALB]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="destination_ip", path="targetIp"),
        DataModelMapping(name="source_ip", path="clientIp"),
        DataModelMapping(name="user_agent", path="userAgent"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

osquery_outdated_agent_tests: list[RuleTest] = [
    RuleTest(
        name="osquery out of date",
        expected_result=True,
        log={
            "action": "added",
            "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
            "columns": {
                "build_distro": "10.12",
                "build_platform": "darwin",
                "config_hash": "1111",
                "config_valid": "1",
                "counter": "14",
                "global_state": "0",
                "extensions": "active",
                "instance_id": "1111",
                "pid": "223",
                "resident_size": "54894592",
                "start_time": "1536634519",
                "system_time": "12472",
                "user_time": "31800",
                "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                "version": "3.1.2",
                "watcher": "92",
            },
            "counter": "255",
            "decorations": {"host_uuid": "1111", "environment": "corp"},
            "epoch": "0",
            "hostIdentifier": "test.lan",
            "log_type": "result",
            "name": "pack_it-compliance_osquery_info",
            "unixTime": "1536682461",
        },
    ),
    RuleTest(
        name="osquery up to date",
        expected_result=False,
        log={
            "action": "added",
            "calendarTime": "Tue Sep 11 16:14:21 2018 UTC",
            "columns": {
                "build_distro": "10.12",
                "build_platform": "darwin",
                "config_hash": "1111",
                "config_valid": "1",
                "counter": "14",
                "global_state": "2",
                "extensions": "active",
                "instance_id": "1111",
                "pid": "223",
                "resident_size": "54894592",
                "start_time": "1536634519",
                "system_time": "12472",
                "user_time": "31800",
                "uuid": "37821E12-CC8A-5AA3-A90C-FAB28A5BF8F9",
                "version": "5.10.2",
                "watcher": "92",
            },
            "counter": "255",
            "decorations": {"host_uuid": "1111", "environment": "corp"},
            "epoch": "0",
            "hostIdentifier": "test.lan",
            "log_type": "result",
            "name": "pack_it-compliance_osquery_info",
            "unixTime": "1536682461",
        },
    ),
]


@panther_managed
class OsqueryOutdatedAgent(Rule):
    id = "Osquery.OutdatedAgent-prototype"
    display_name = "Osquery Agent Outdated"
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Osquery", "Compliance"]
    default_severity = Severity.INFO
    default_description = "Keep track of osquery versions, current is 5.10.2."
    default_runbook = "Update the osquery agent."
    default_reference = "https://www.osquery.io/downloads/official/5.10.2"
    summary_attributes = ["name", "hostIdentifier", "action"]
    tests = osquery_outdated_agent_tests
    LATEST_VERSION = "5.10.2"

    def rule(self, event):
        return (
            event.get("name") == "pack_it-compliance_osquery_info"
            and deep_get(event, "columns", "version") != self.LATEST_VERSION
            and (event.get("action") == "added")
        )

    def title(self, event):
        return f"Osquery Version {deep_get(event, 'columns', 'version')} is Outdated"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

one_login_password_access_tests: list[RuleTest] = [
    RuleTest(
        name="User accessed their own password",
        expected_result=False,
        log={
            "event_type_id": "240",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    RuleTest(
        name="User accessed another user's password",
        expected_result=True,
        log={
            "event_type_id": "240",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


@panther_managed
class OneLoginPasswordAccess(Rule):
    id = "OneLogin.PasswordAccess-prototype"
    display_name = "OneLogin Password Access"
    log_types = [LogType.ONELOGIN_EVENTS]
    tags = ["OneLogin", "Credential Access:Unsecured Credentials"]
    reports = {"MITRE ATT&CK": ["TA0006:T1552"]}
    default_severity = Severity.MEDIUM
    default_description = "User accessed another user's application password\n"
    default_reference = "https://onelogin.service-now.com/kb_view_customer.do?sysparm_article=KB0010598"
    default_runbook = "Investigate whether this was authorized access.\n"
    summary_attributes = ["account_id", "user_name", "user_id"]
    tests = one_login_password_access_tests

    def rule(self, event):
        # Filter events; event type 240 is actor_user revealed user's app password
        if str(event.get("event_type_id")) != "240" or not event.get("actor_user_id") or (not event.get("user_id")):
            return False
        # Determine if actor_user accessed another user's password
        return event.get("actor_user_id") != event.get("user_id")

    def dedup(self, event):
        return event.get("actor_user_name") + ":" + event.get("app_name", "<UNKNOWN_APP>")

    def title(self, event):
        return f"A user [{event.get('actor_user_name', '<UNKNOWN_USER>')}] accessed another user's [{event.get('user_name', '<UNKNOWN_USER>')}] [{event.get('app_name', '<UNKNOWN_APP>')}] password"



"""Global helpers for MongoDB detections"""


def mongodb_alert_context(event) -> dict:
    return {
        "username": event.get("username", "<USER_NOT_FOUND>"),
        "target_username": event.get("targetUsername", "<USER_NOT_FOUND>"),
        "org_id": event.get("orgId", "<ORG_NOT_FOUND>"),
        "remote_address": event.get("remoteAddress", "<REMOTE_ADDRESS_NOT_FOUND>"),
    }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_suspicious_logins_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Login Event",
        expected_result=False,
        log={
            "id": {"applicationName": "login"},
            "kind": "admin#reports#activity",
            "type": "account_warning",
            "name": "login_success",
            "parameters": {"affected_email_address": "bobert@ext.runpanther.io"},
        },
    ),
    RuleTest(
        name="Account Warning Not For Suspicious Login",
        expected_result=False,
        log={
            "id": {"applicationName": "login"},
            "kind": "admin#reports#activity",
            "type": "account_warning",
            "name": "account_disabled_spamming",
            "parameters": {"affected_email_address": "bobert@ext.runpanther.io"},
        },
    ),
    RuleTest(
        name="Account Warning For Suspicious Login",
        expected_result=True,
        log={
            "id": {"applicationName": "login"},
            "kind": "admin#reports#activity",
            "type": "account_warning",
            "name": "suspicious_login",
            "parameters": {"affected_email_address": "bobert@ext.runpanther.io"},
        },
    ),
]


@panther_managed
class GSuiteSuspiciousLogins(Rule):
    id = "GSuite.SuspiciousLogins-prototype"
    display_name = "Suspicious GSuite Login"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.MEDIUM
    default_description = "GSuite reported a suspicious login for this user.\n"
    default_reference = "https://support.google.com/a/answer/7102416?hl=en"
    default_runbook = "Checkout the details of the login and verify this behavior with the user to ensure the account wasn't compromised.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_suspicious_logins_tests
    SUSPICIOUS_LOGIN_TYPES = {"suspicious_login", "suspicious_login_less_secure_app", "suspicious_programmatic_login"}

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "login":
            return False
        return bool(event.get("name") in self.SUSPICIOUS_LOGIN_TYPES)

    def title(self, event):
        user = deep_get(event, "parameters", "affected_email_address")
        if not user:
            user = "<UNKNOWN_USER>"
        return f"A suspicious login was reported for user [{user}]"



import inspect
import json
from dataclasses import dataclass, field
from typing import Any

from panther_core.detection import DetectionResult

from pypanther.severity import Severity
from pypanther.utils import try_asdict

RULE_TEST_ALL_ATTRS = [
    "name",
    "expected_result",
    "log",
    "mocks",
    "expected_severity",
    "expected_title",
    "expected_dedup",
    "expected_runbook",
    "expected_reference",
    "expected_description",
    "expected_alert_context",
]

RULE_MOCK_ALL_ATTRS = [
    "object_name",
    "return_value",
    "side_effect",
]

RULE_TEST_RESULT_ALL_ATTRS = [
    "passed",
    "detection_result",
    "test",
    "rule_id",
]


@dataclass
class RuleMock:
    object_name: str
    return_value: Any = None
    side_effect: Any = None
    new: Any = None

    def asdict(self):
        """Returns a dictionary representation of the class."""
        return {key: try_asdict(getattr(self, key)) for key in RULE_MOCK_ALL_ATTRS}


class FileLocationMeta(type):
    def __call__(cls, *args, **kwargs):
        frame = inspect.currentframe().f_back
        file_path = frame.f_globals.get("__file__", None)
        line_number = frame.f_lineno
        module = frame.f_globals.get("__name__", None)
        instance = super().__call__(*args, **kwargs, _file_path=file_path, _line_no=line_number, _module=module)
        return instance


@dataclass
class RuleTest(metaclass=FileLocationMeta):
    name: str
    expected_result: bool
    log: dict | str
    mocks: list[RuleMock] = field(default_factory=list)
    expected_severity: Severity | str | None = None
    expected_title: str | None = None
    expected_dedup: str | None = None
    expected_runbook: str | None = None
    expected_reference: str | None = None
    expected_description: str | None = None
    expected_alert_context: dict | None = None
    # expected_destinations is not included here because the `destinations` function
    # checks a list of valid destinations to check if the destination exists, and if it is
    # the name of the destination, it gets the id of it. pypanther does not provide support
    # for supplying that list to tests yet so this aux check is excluded.
    _file_path: str = ""
    _line_no: int = 0
    _module: str = ""

    def log_data(self):
        if isinstance(self.log, str):
            return json.loads(self.log)
        return self.log

    def location(self) -> str:
        return f"{self._file_path}:{self._line_no}"

    def asdict(self):
        """Returns a dictionary representation of the class."""
        return {key: try_asdict(getattr(self, key)) for key in RULE_TEST_ALL_ATTRS}


@dataclass
class RuleTestResult:
    """
    PantherRuleTestResult is the output returned from running a PantherRuleTest
    on a PantherRule.

    Attributes
    ----------
        passed: If true, the PantherRuleTest passed. False, otherwise.
        detection_result: The result of the run() function on the given PantherEvent.
        test: The test that was given and created this result.
        rule_id: The ID of the PantherRule the PantherRuleTest was run on.

    """

    passed: bool
    detection_result: DetectionResult
    test: RuleTest
    rule_id: str



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context

awss3_server_access_unauthenticated_tests: list[RuleTest] = [
    RuleTest(
        name="Authenticated Access",
        expected_result=False,
        log={
            "bucket": "example-bucket",
            "requester": "79a59df900b949e55d96a1e698fbacedfd6e09d98eacf8f8d5218e7cd47ef2be",
        },
    ),
    RuleTest(name="Unauthenticated Access", expected_result=True, log={"bucket": "example-bucket"}),
]


@panther_managed
class AWSS3ServerAccessUnauthenticated(Rule):
    id = "AWS.S3.ServerAccess.Unauthenticated-prototype"
    display_name = "AWS S3 Unauthenticated Access"
    enabled = False
    log_types = [LogType.AWS_S3_SERVER_ACCESS]
    tags = ["AWS", "Configuration Required", "Security Control", "Collection:Data From Cloud Storage Object"]
    reports = {"MITRE ATT&CK": ["TA0009:T1530"]}
    default_severity = Severity.LOW
    default_description = "Checks for S3 access attempts where the requester is not an authenticated AWS user.\n"
    default_runbook = "If unauthenticated S3 access is not expected for this bucket, update its access policies.\n"
    default_reference = (
        "https://docs.aws.amazon.com/AmazonS3/latest/userguide/access-control-auth-workflow-bucket-operation.html"
    )
    summary_attributes = ["bucket", "key", "requester"]
    tests = awss3_server_access_unauthenticated_tests
    # A list of buckets where authenticated access is expected
    AUTH_BUCKETS = {"example-bucket"}

    def rule(self, event):
        return event.get("bucket") in self.AUTH_BUCKETS and (not event.get("requester"))

    def title(self, event):
        return f"Unauthenticated access to S3 Bucket [{event.get('bucket', '<UNKNOWN_BUCKET')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import okta_alert_context

okta_user_mfa_reset_single_tests: list[RuleTest] = [
    RuleTest(
        name="User reset own MFA factor",
        expected_result=True,
        log={
            "eventtype": "user.mfa.factor.deactivate",
            "version": "0",
            "severity": "INFO",
            "displaymessage": "Reset factor for user",
            "actor": {
                "alternateId": "homer@springfield.gov",
                "displayName": "Homer Simpson",
                "id": "11111111111",
                "type": "User",
            },
            "client": {
                "device": "Computer",
                "ipAddress": "1.1.1.1",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/96.0.4664.55 Safari/537.36",
                },
                "zone": "null",
            },
            "outcome": {"reason": "User reset FIDO_WEBAUTHN factor", "result": "SUCCESS"},
            "target": [
                {
                    "alternateId": "homer@springfield.gov",
                    "displayName": "Homer Simpson",
                    "id": "1111111",
                    "type": "User",
                },
            ],
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "1111111"},
            "p_log_type": "Okta.SystemLog",
        },
    ),
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "p_log_type": "Okta.SystemLog",
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc456",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "abc12345"},
            "client": {
                "device": "Unknown",
                "ipAddress": "1.2.3.4",
                "userAgent": {"browser": "UNKNOWN", "os": "Unknown", "rawUserAgent": "Chrome"},
                "zone": "null",
            },
            "debugcontext": {"debugData": {}},
            "eventtype": "application.integration.rate_limit_exceeded",
            "legacyeventtype": "app.api.error.rate.limit.exceeded",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-06-10 17:19:58.423",
            "request": {},
            "securitycontext": {},
            "severity": "INFO",
            "target": [{"alternateId": "App ", "displayName": "App", "id": "12345", "type": "AppInstance"}],
            "transaction": {"detail": {}, "id": "sdfg", "type": "JOB"},
            "uuid": "aaa-bb-ccc",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaUserMFAResetSingle(Rule):
    default_description = "User has reset one of their own MFA factors"
    display_name = "Okta User MFA Own Reset"
    id = "Okta.User.MFA.Reset.Single-prototype"
    default_reference = "https://support.okta.com/help/s/article/How-to-avoid-lockouts-and-reset-your-Multifactor-Authentication-MFA-for-Okta-Admins?language=en_US"
    default_severity = Severity.INFO
    log_types = [LogType.OKTA_SYSTEM_LOG]
    tests = okta_user_mfa_reset_single_tests

    def rule(self, event):
        return event.udm("event_type") == event_type.MFA_RESET

    def title(self, event):
        try:
            which_factor = event.get("outcome", {}).get("reason", "").split()[2]
        except IndexError:
            which_factor = "<FACTOR_NOT_FOUND>"
        return f"Okta: User reset their MFA factor [{which_factor}] [{event.get('target', [{}])[0].get('alternateId', '<id-not-found>')}] by [{event.get('actor', {}).get('alternateId', '<id-not-found>')}]"

    def alert_context(self, event):
        return okta_alert_context(event)



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType

audit_log_type_map = {
    "user_login_succeeded": event_type.SUCCESSFUL_LOGIN,
    "user_login_failed": event_type.FAILED_LOGIN,
    "user_invited": event_type.USER_ACCOUNT_CREATED,
    "user_reprovisioned": event_type.USER_ACCOUNT_CREATED,
    "user_deprovisioned": event_type.USER_ACCOUNT_DELETED,
    "user_workspace_admin_role_changed": event_type.ADMIN_ROLE_ASSIGNED,
}


def get_event_type(event):
    logged_event_type = event.get("event_type", {})
    # Since this is a safe dict get if the event type is not mapped
    # there is an implicit return of None
    return audit_log_type_map.get(logged_event_type)


class StandardAsanaAudit(DataModel):
    id: str = "Standard.Asana.Audit"
    display_name: str = "Asana Audit Logs"
    enabled: bool = True
    log_types: list[str] = [LogType.ASANA_AUDIT]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="$.actor.name"),
        DataModelMapping(name="event_type", method=get_event_type),
        DataModelMapping(name="source_ip", path="$.context.client_ip_address"),
        DataModelMapping(name="user", path="$.resource.name"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context

snyk_system_policy_setting_tests: list[RuleTest] = [
    RuleTest(
        name="Snyk System Policy Setting event happened ( Security Policy )",
        expected_result=True,
        log={
            "content": {
                "after": {
                    "configuration": {
                        "rules": [
                            {
                                "actions": [{"data": {"severity": "high"}, "type": "severity-override"}],
                                "conditions": {
                                    "AND": [{"field": "exploit-maturity", "operator": "includes", "value": ["mature"]}],
                                },
                                "name": "Rule 1",
                            },
                        ],
                    },
                    "description": "This is a security policy",
                    "group": "8fffffff-1555-4444-b000-b55555555555",
                    "name": "Example Security Policy",
                },
                "before": {},
                "publicId": "21111111-a222-4eee-8ddd-a99999999999",
            },
            "created": "2023-03-03 00:13:45.497",
            "event": "group.policy.create",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk System Policy Setting event happened ( License Policy )",
        expected_result=True,
        log={
            "content": {
                "after": {
                    "configuration": {
                        "licenses": [
                            {"instructions": "", "licenseType": "ADSL", "severity": "medium"},
                            {"instructions": "", "licenseType": "AGPL-3.0", "severity": "medium"},
                            {"instructions": "", "licenseType": "AGPL-3.0-only", "severity": "high"},
                        ],
                    },
                    "description": "this is a policy description",
                    "group": "8fffffff-1555-4444-b000-b55555555555",
                    "name": "Example License Policy",
                    "projectAttributes": {"criticality": [], "environment": [], "lifecycle": []},
                },
                "before": {},
                "publicId": "21111111-a222-4eee-8ddd-a99999999999",
            },
            "created": "2023-03-03 00:10:02.351",
            "event": "group.policy.create",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk Group SSO Membership sync",
        expected_result=False,
        log={
            "content": {
                "addAsOrgAdmin": [],
                "addAsOrgCollaborator": ["group.name"],
                "addAsOrgCustomRole": [],
                "addAsOrgRestrictedCollaborator": [],
                "removedOrgMemberships": [],
                "userPublicId": "05555555-3333-4ddd-8ccc-755555555555",
            },
            "created": "2023-03-15 13:13:13.133",
            "event": "group.sso.membership.sync",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
        },
    ),
]


@panther_managed
class SnykSystemPolicySetting(Rule):
    id = "Snyk.System.PolicySetting-prototype"
    display_name = "Snyk System Policy Settings Changed"
    log_types = [LogType.SNYK_GROUP_AUDIT, LogType.SNYK_ORG_AUDIT]
    tags = ["Snyk"]
    default_severity = Severity.HIGH
    default_description = "Detects Snyk Policy Settings have been changed. Policies define Snyk's behavior when encountering security and licensing issues.\n"
    default_runbook = "Snyk Policies can cause alerts to raise or not based on found security and license issues. Validate that that this change is expected.\n"
    default_reference = "https://docs.snyk.io/manage-issues/policies/shared-policies-overview"
    summary_attributes = ["event"]
    tests = snyk_system_policy_setting_tests
    ACTIONS = [
        "group.policy.create",
        "group.policy.delete",
        "group.policy.edit",
        "org.policy.edit",
        "org.ignore_policy.edit",
    ]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        return action in self.ACTIONS

    def title(self, event):
        policy_type = "<NO_POLICY_TYPE_FOUND>"
        license_or_rule = deep_get(event, "content", "after", "configuration", default={})
        if "rules" in license_or_rule:
            policy_type = "security"
        elif "licenses" in license_or_rule:
            policy_type = "license"
        return f"Snyk: System [{policy_type}] Policy Setting event [{deep_get(event, 'event', default='<NO_EVENT>')}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        a_c = snyk_alert_context(event)
        a_c["policy_type"] = "<NO_POLICY_TYPE_FOUND>"
        license_or_rule = deep_get(event, "content", "after", "configuration", default={})
        if "rules" in license_or_rule:
            a_c["policy_type"] = "security"
        elif "licenses" in license_or_rule:
            a_c["policy_type"] = "license"
        return a_c

    def dedup(self, event):
        # Licenses can apply at org or group levels
        return f"{deep_get(event, 'userId', default='<NO_USERID>')}{deep_get(event, 'orgId', default='<NO_ORGID>')}{deep_get(event, 'groupId', default='<NO_GROUPID>')}{deep_get(event, 'content', 'publicId', default='<NO_PUBLICID>')}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

one_login_password_changed_tests: list[RuleTest] = [
    RuleTest(
        name="User changed their password",
        expected_result=True,
        log={
            "event_type_id": "11",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    RuleTest(
        name="User changed another's password",
        expected_result=True,
        log={
            "event_type_id": "11",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    RuleTest(
        name="Admin user changed another's password",
        expected_result=False,
        log={
            "event_type_id": "211",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


@panther_managed
class OneLoginPasswordChanged(Rule):
    id = "OneLogin.PasswordChanged-prototype"
    display_name = "OneLogin User Password Changed"
    log_types = [LogType.ONELOGIN_EVENTS]
    tags = ["OneLogin", "Identity & Access Management"]
    default_severity = Severity.INFO
    default_description = "A user password was updated.\n"
    default_reference = "https://onelogin.service-now.com/kb_view_customer.do?sysparm_article=KB0010510"
    default_runbook = "Investigate whether this was an authorized action.\n"
    summary_attributes = ["account_id", "user_name", "user_id"]
    tests = one_login_password_changed_tests

    def rule(self, event):
        # check that this is a password change event;
        # event id 11 is actor_user changed password for user
        # Normally, admin's may change a user's password (event id 211)
        return str(event.get("event_type_id")) == "11"

    def title(self, event):
        return f"A user [{event.get('user_name', '<UNKNOWN_USER>')}] password changed by user [{event.get('actor_user_name', '<UNKNOWN_USER>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context

snyk_role_change_tests: list[RuleTest] = [
    RuleTest(
        name="Snyk Group Role Edit -  ADMIN role ( CRIT )",
        expected_result=True,
        log={
            "content": {
                "after": {"role": "ADMIN", "rolePublicId": "8ddddddd-fbbb-4fff-8111-5eeeeeeeeeee"},
                "before": {"role": "MEMBER", "rolePublicId": "6aaaaaaa-c000-4ddd-9ddd-c55555555555"},
                "userPublicId": "05555555-3333-4ddd-8ccc-755555555555",
            },
            "created": "1999-04-04 18:38:19.843",
            "event": "group.user.role.edit",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk Org Role Edit -  ADMIN role ( CRIT )",
        expected_result=True,
        log={
            "content": {
                "afterRoleName": "ADMIN",
                "afterRolePublicId": "d8999999-aaaa-4999-9f0b-9bbbbbbbbbbb",
                "beforeRoleName": "Org Collaborator",
                "beforeRolePublicId": "b0000000-dbbb-4000-addd-1ddddddddddd",
                "userPublicId": "05555555-3333-4ddd-8ccc-755555555555",
            },
            "created": "1999-03-08 16:24:02.616",
            "event": "org.user.role.edit",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
        },
    ),
    RuleTest(
        name="Snyk Group Role Edit -  MEMBER role ( MEDIUM )",
        expected_result=True,
        log={
            "content": {
                "before": {"role": "ADMIN", "rolePublicId": "8ddddddd-fbbb-4fff-8111-5eeeeeeeeeee"},
                "after": {"role": "MEMBER", "rolePublicId": "6aaaaaaa-c000-4ddd-9ddd-c55555555555"},
                "userPublicId": "05555555-3333-4ddd-8ccc-755555555555",
            },
            "created": "1999-04-04 18:38:19.843",
            "event": "group.user.role.edit",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "userId": "05555555-3333-4ddd-8ccc-755555555555",
        },
    ),
    RuleTest(
        name="Snyk Group SSO Membership sync",
        expected_result=False,
        log={
            "content": {},
            "created": "2023-03-15 13:13:13.133",
            "event": "group.sso.membership.sync",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
        },
    ),
]


@panther_managed
class SnykRoleChange(Rule):
    id = "Snyk.Role.Change-prototype"
    display_name = "Snyk Role Change"
    log_types = [LogType.SNYK_GROUP_AUDIT, LogType.SNYK_ORG_AUDIT]
    tags = ["Snyk"]
    default_severity = Severity.HIGH
    default_description = "Detects when Snyk Roles are changed\n"
    default_runbook = "These actions in the Snyk Audit logs indicate that a ServiceAccount has been created/deleted/modified.\nAll events where the Role is marked as ADMIN have CRITICAL severity Other events are marked with MEDIUM severity\n"
    default_reference = "https://docs.snyk.io/snyk-admin/manage-users-and-permissions/member-roles"
    summary_attributes = ["event"]
    tests = snyk_role_change_tests
    ACTIONS = [
        "group.role.create",
        "group.role.edit",
        "group.user.role.create",
        "group.user.role.delete",
        "group.user.role.edit",
        "org.user.role.create",
        "org.user.role.delete",
        "org.user.role.details.edit",
        "org.user.role.edit",
        "org.user.role.permissions.edit",
    ]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        return action in self.ACTIONS

    def title(self, event):
        group_or_org = "<GROUP_OR_ORG>"
        crud_operation = "<NO_OPERATION>"
        action = deep_get(event, "event", default="<NO_EVENT>")
        if "." in action:
            group_or_org = action.split(".")[0].title()
            crud_operation = action.split(".")[-1].title()
        return f"Snyk: [{group_or_org}] Role [{crud_operation}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        a_c = snyk_alert_context(event)
        role = deep_get(event, "content", "after", "role", default=None)
        if not role and "afterRoleName" in deep_get(event, "content", default={}):
            role = deep_get(event, "content", "afterRoleName", default=None)
        if role:
            a_c["role_permission"] = role
        return a_c

    def dedup(self, event):
        return f"{deep_get(event, 'userId', default='<NO_USERID>')}{deep_get(event, 'orgId', default='<NO_ORGID>')}{deep_get(event, 'groupId', default='<NO_GROUPID>')}{deep_get(event, 'event', default='<NO_EVENT>')}"

    def severity(self, event):
        role = deep_get(event, "content", "after", "role", default=None)
        if not role and "afterRoleName" in deep_get(event, "content", default={}):
            role = deep_get(event, "content", "afterRoleName", default=None)
        if role == "ADMIN":
            return "CRITICAL"
        return "MEDIUM"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import slack_alert_context

slack_audit_logs_idp_configuration_changed_tests: list[RuleTest] = [
    RuleTest(
        name="IDP Configuration Added",
        expected_result=True,
        log={
            "action": "idp_configuration_added",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 16:48:14",
        },
    ),
    RuleTest(
        name="IDP Configuration Deleted",
        expected_result=True,
        log={
            "action": "idp_configuration_deleted",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 16:48:14",
        },
    ),
    RuleTest(
        name="IDP Configuration Updated",
        expected_result=True,
        log={
            "action": "idp_prod_configuration_updated",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 16:48:14",
        },
    ),
    RuleTest(
        name="User Logout",
        expected_result=False,
        log={
            "action": "user_logout",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace-1",
                    "id": "T01234N56GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 15:22:32",
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
        },
    ),
]


@panther_managed
class SlackAuditLogsIDPConfigurationChanged(Rule):
    id = "Slack.AuditLogs.IDPConfigurationChanged-prototype"
    display_name = "Slack IDP Configuration Changed"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Persistence", "Credential Access", "Modify Authentication Process"]
    reports = {"MITRE ATT&CK": ["TA0003:T1556", "TA0006:T1556"]}
    default_severity = Severity.HIGH
    default_description = "Detects changes to the identity provider (IdP) configuration for Slack organizations."
    default_reference = "https://slack.com/intl/en-gb/help/articles/115001435788-Connect-identity-provider-groups-to-your-Enterprise-Grid-org"
    summary_attributes = ["action", "p_any_ip_addresses", "p_any_emails"]
    tests = slack_audit_logs_idp_configuration_changed_tests
    IDP_CHANGE_ACTIONS = {
        "idp_configuration_added": "Slack IDP Configuration Added",
        "idp_configuration_deleted": "Slack IDP Configuration Deleted",
        "idp_prod_configuration_updated": "Slack IDP Configuration Updated",
    }

    def rule(self, event):
        return event.get("action") in self.IDP_CHANGE_ACTIONS

    def title(self, event):
        if event.get("action") in self.IDP_CHANGE_ACTIONS:
            return self.IDP_CHANGE_ACTIONS.get(event.get("action"))
        return "Slack IDP Configuration Changed"

    def alert_context(self, event):
        return slack_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

teleport_root_login_tests: list[RuleTest] = [
    RuleTest(
        name="User logged in as root",
        expected_result=True,
        log={
            "addr_remote": "192.0.2.1:35194",
            "cluster_name": "teleport.example.com",
            "code": "t2000i",
            "ei": 0,
            "event": "session.start",
            "initial_command": [""],
            "login": "root",
            "namespace": "default",
            "proto": "ssh",
            "server_hostname": "ip-10-0-0-1.us-west-2.compute.internal",
            "server_id": "12345678-1111-2222-3333-54e9467ff0e6",
            "server_labels": {"env": "prod", "role": "project123"},
            "session_recording": "node-sync",
            "sid": "12345678-1111-2222-3333-54e9467ff0e6",
            "size": "80:25",
            "time": "2023-09-18 11:22:33",
            "uid": "12345678-1111-2222-3333-54e9467ff0e6",
            "user": "max.mustermann@zumbeispiel.example",
        },
    ),
]


@panther_managed
class TeleportRootLogin(Rule):
    id = "Teleport.RootLogin-prototype"
    display_name = "User Logged in as root"
    log_types = [LogType.GRAVITATIONAL_TELEPORT_AUDIT]
    tags = ["SSH", "Execution:Command and Scripting Interpreter", "Teleport"]
    default_severity = Severity.MEDIUM
    default_description = "A User logged in as root"
    reports = {"MITRE ATT&CK": ["TA0002:T1059"]}
    default_reference = "https://goteleport.com/docs/management/admin/"
    default_runbook = "Use user accounts and policies, rather than root when logging in. With access to root, it is possible to evade auditing and logging.\n"
    summary_attributes = ["event", "code", "user", "login", "server_hostname", "server_labels"]
    tests = teleport_root_login_tests

    def rule(self, event):
        return event.get("event") == "session.start" and event.get("login") == "root"

    def title(self, event):
        return f"User [{event.get('user', '<UNKNOWN_USER>')}] logged into [{event.get('server_hostname', '<UNKNOWN_HOSTNAME>')}] as root on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, okta_alert_context

okta_api_key_created_tests: list[RuleTest] = [
    RuleTest(
        name="API Key Created",
        expected_result=True,
        log={
            "uuid": "2a992f80-d1ad-4f62-900e-8c68bb72a21b",
            "published": "2021-01-08 21:28:34.875",
            "eventType": "system.api_token.create",
            "version": "0",
            "severity": "INFO",
            "legacyEventType": "api.token.create",
            "displayMessage": "Create API token",
            "actor": {
                "alternateId": "user@example.com",
                "displayName": "Test User",
                "id": "00u3q14ei6KUOm4Xi2p4",
                "type": "User",
            },
            "outcome": {"result": "SUCCESS"},
            "request": {},
            "debugContext": {},
            "target": [
                {
                    "id": "00Tpki36zlWjhjQ1u2p4",
                    "type": "Token",
                    "alternateId": "unknown",
                    "displayName": "test_key",
                    "details": None,
                },
            ],
        },
    ),
]


@panther_managed
class OktaAPIKeyCreated(Rule):
    id = "Okta.APIKeyCreated-prototype"
    display_name = "Okta API Key Created"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    tags = ["Identity & Access Management", "Okta", "Credential Access:Steal Application Access Token"]
    reports = {"MITRE ATT&CK": ["TA0006:T1528"]}
    default_severity = Severity.INFO
    default_description = "A user created an API Key in Okta"
    default_reference = "https://help.okta.com/en/prod/Content/Topics/Security/API.htm"
    default_runbook = "Reach out to the user if needed to validate the activity."
    summary_attributes = ["eventType", "severity", "displayMessage", "p_any_ip_addresses"]
    tests = okta_api_key_created_tests

    def rule(self, event):
        return (
            event.get("eventType", None) == "system.api_token.create"
            and deep_get(event, "outcome", "result") == "SUCCESS"
        )

    def title(self, event):
        target = event.get("target", [{}])
        key_name = target[0].get("displayName", "MISSING DISPLAY NAME") if target else "MISSING TARGET"
        return f"{deep_get(event, 'actor', 'displayName')} <{deep_get(event, 'actor', 'alternateId')}>created a new API key - <{key_name}>"

    def alert_context(self, event):
        return okta_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

zoom_all_meetings_secured_with_one_option_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="Turn off",
        expected_result=True,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Require that all meetings are secured with one security option: from On to Off",
            "operator": "example@example.io",
            "time": "2022-12-16 18:15:38",
        },
    ),
    RuleTest(
        name="Turn on",
        expected_result=False,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Require that all meetings are secured with one security option: from Off to On",
            "operator": "example@example.io",
            "time": "2022-12-16 18:15:38",
        },
    ),
    RuleTest(
        name="Non admin user update",
        expected_result=False,
        log={
            "action": "Update",
            "category_type": "User",
            "operation_detail": "Update User example@example.io  - Job Title: set to Contractor",
            "operator": "homer@example.io",
        },
    ),
]


@panther_managed
class ZoomAllMeetingsSecuredWithOneOptionDisabled(Rule):
    default_description = (
        "A Zoom User turned off your organization's requirement that all meetings are secured with one security option."
    )
    display_name = "Zoom All Meetings Secured With One Option Disabled"
    default_runbook = (
        "Confirm this user acted with valid business intent and determine whether this activity was authorized."
    )
    default_reference = "https://support.zoom.com/hc/en/article?id=zm_kb&sysparm_article=KB0059862"
    default_severity = Severity.MEDIUM
    log_types = [LogType.ZOOM_OPERATION]
    id = "Zoom.All.Meetings.Secured.With.One.Option.Disabled-prototype"
    tests = zoom_all_meetings_secured_with_one_option_disabled_tests

    def rule(self, event):
        operation_detail = event.get("operation_detail", "<NO_OPS_DETAIL>")
        operation_flag = "Require that all meetings are secured with one security option: from On to Off"
        return (
            event.get("action", "<NO_ACTION>") == "Update"
            and event.get("category_type", "<NO_CATEGORY_TYPE>") == "Account"
            and (operation_flag in operation_detail)
        )

    def title(self, event):
        return f"Zoom User [{event.get('operator', '<NO_OPERATOR>')}] turned off your organization's requirement to secure all meetings with one security option."



import json
from unittest.mock import MagicMock

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, github_alert_context

git_hub_action_failed_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - Branch Protection Disabled",
        expected_result=False,
        log={
            "actor": "cat",
            "action": "protected_branch.destroy",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
    RuleTest(
        name="GitHub Action Failed - No Configuration",
        expected_result=False,
        log={
            "_document_id": "pWWWWWWWWWWWWWWWWWWWWW",
            "action": "workflows.completed_workflow_run",
            "actor": "github_handle",
            "at_sign_timestamp": "2023-01-31 18:58:27.638",
            "business": "github-business-only-if-enterprise-audit-log",
            "completed_at": "2023-01-31T18:58:27.000Z",
            "conclusion": "failure",
            "created_at": "2023-01-31 18:58:27.638",
            "event": "schedule",
            "head_branch": "master",
            "head_sha": "66dddddddddddddddddddddddddddddddddddddd",
            "name": "sync-panther-analysis-from-upstream",
            "operation_type": "modify",
            "org": "panther-labs",
            "p_log_type": "GitHub.Audit",
            "public_repo": False,
            "repo": "your-org/panther-analysis-copy",
            "run_attempt": 3,
            "run_number": 99,
            "started_at": "2023-01-31 18:58:04",
            "workflow_id": 44444444,
            "workflow_run_id": 5555555555,
        },
    ),
    RuleTest(
        name="GitHub Action Failed - Monitored Action Configured",
        expected_result=True,
        mocks=[
            RuleMock(
                object_name="MONITORED_ACTIONS",
                return_value='{\n  "your-org/panther-analysis-copy": [ "sync-panther-analysis-from-upstream"]\n}',
            ),
        ],
        log={
            "_document_id": "pWWWWWWWWWWWWWWWWWWWWW",
            "action": "workflows.completed_workflow_run",
            "actor": "github_handle",
            "at_sign_timestamp": "2023-01-31 18:58:27.638",
            "business": "github-business-only-if-enterprise-audit-log",
            "completed_at": "2023-01-31T18:58:27.000Z",
            "conclusion": "failure",
            "created_at": "2023-01-31 18:58:27.638",
            "event": "schedule",
            "head_branch": "master",
            "head_sha": "66dddddddddddddddddddddddddddddddddddddd",
            "name": "sync-panther-analysis-from-upstream",
            "operation_type": "modify",
            "org": "panther-labs",
            "p_log_type": "GitHub.Audit",
            "public_repo": False,
            "repo": "your-org/panther-analysis-copy",
            "run_attempt": 3,
            "run_number": 99,
            "started_at": "2023-01-31 18:58:04",
            "workflow_id": 44444444,
            "workflow_run_id": 5555555555,
        },
    ),
]


@panther_managed
class GitHubActionFailed(Rule):
    id = "GitHub.Action.Failed-prototype"
    display_name = "GitHub Action Failed"
    enabled = False
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Configuration Required"]
    default_severity = Severity.HIGH
    default_description = "A monitored github action has failed."
    default_runbook = "Inspect the action failure link and take appropriate response. There are no general plans of response for this activity.\n"
    default_reference = (
        "https://docs.github.com/en/actions/creating-actions/setting-exit-codes-for-actions#about-exit-codes"
    )
    tests = git_hub_action_failed_tests
    # The keys for MONITORED_ACTIONS are gh_org/repo_name
    # The values for MONITORED_ACTIONS are a list of ["action_names"]
    MONITORED_ACTIONS = {}

    def rule(self, event):
        if isinstance(self.MONITORED_ACTIONS, MagicMock):
            self.MONITORED_ACTIONS = json.loads(self.MONITORED_ACTIONS())  # pylint: disable=not-callable
        repo = deep_get(event, "repo", default="")
        action_name = deep_get(event, "name", default="")
        return all(
            [
                deep_get(event, "action", default="") == "workflows.completed_workflow_run",
                repo in self.MONITORED_ACTIONS,
                action_name in self.MONITORED_ACTIONS.get(repo, []),
                deep_get(event, "conclusion", default="") == "failure",
            ],
        )

    def title(self, event):
        repo = deep_get(event, "repo", default="<NO_REPO>")
        action_name = deep_get(event, "name", default="<NO_ACTION_NAME>")
        return f"The GitHub Action [{action_name}] in [{repo}] has failed"

    def alert_context(self, event):
        a_c = github_alert_context(event)
        a_c["action"] = event.get("name", "<NO_ACTION_NAME>")
        a_c["action_run_link"] = (
            f"https://github.com/{a_c.get('repo')}/actions/runs/{event.get('workflow_run_id', '<NO_RUN_ID>')}"
        )
        return a_c



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_config import config

box_event_triggered_externally_tests: list[RuleTest] = [
    RuleTest(
        name="Regular Event",
        expected_result=False,
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": "12345678", "type": "user", "login": "cat@example.com", "name": "Bob Cat"},
            "event_type": "DELETE",
        },
    ),
    RuleTest(
        name="Previewed Anonymously",
        expected_result=True,
        log={
            "created_by": {"id": "2", "type": "user", "name": "Unknown User"},
            "event_type": "PREVIEW",
            "type": "event",
            "ip_address": "1.2.3.4",
        },
    ),
    RuleTest(
        name="Missing Created By",
        expected_result=False,
        log={"event_type": "PREVIEW", "type": "event", "ip_address": "1.2.3.4"},
    ),
]


@panther_managed
class BoxEventTriggeredExternally(Rule):
    id = "Box.Event.Triggered.Externally-prototype"
    display_name = "Box event triggered by unknown or external user"
    enabled = False
    log_types = [LogType.BOX_EVENT]
    tags = ["Box", "Exfiltration:Exfiltration Over Web Service", "Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0010:T1567"]}
    default_severity = Severity.MEDIUM
    default_description = "An external user has triggered a box enterprise event.\n"
    default_reference = "https://support.box.com/hc/en-us/articles/8391393127955-Using-the-Enterprise-Event-Stream"
    default_runbook = "Investigate whether this user's activity is expected.\n"
    summary_attributes = ["ip_address"]
    threshold = 10
    tests = box_event_triggered_externally_tests
    DOMAINS = {"@" + domain for domain in config.ORGANIZATION_DOMAINS}

    def rule(self, event):
        # Check that all events are triggered by internal users
        if event.get("event_type") not in ("FAILED_LOGIN", "SHIELD_ALERT"):
            user = event.get("created_by", {})
            # user id 2 indicates an anonymous user
            if user.get("id", "") == "2":
                return True
            return bool(user.get("login") and (not any(user.get("login", "").endswith(x) for x in self.DOMAINS)))
        return False

    def title(self, event):
        return (
            f"External user [{deep_get(event, 'created_by', 'login', default='<UNKNOWN_USER>')}] triggered a box event."
        )



"""Used to define global helpers for Zoom events"""


def get_zoom_user_context(event):
    """
    Parses the operation_detail field of Zoom.Operation events related to Users
    to provide usable fields for use in detections
    """
    operation_context = {}
    raw_string = event.get("operation_detail", "")
    category_type = event.get("category_type")
    action = event.get("action")

    if category_type == "User":
        if action in ("Add", "Delete"):
            operation_context["User"] = raw_string.split()[2].strip()
            operation_context["Department"] = raw_string.split("-")[2].split(":")[1].strip()
            operation_context["UserType"] = raw_string.split("-")[1].split(":")[1].strip()

        if action == "Update":
            operation_context["User"] = raw_string.split()[2]
            operation_context["Change"] = " ".join(raw_string.split("-")).strip()
            operation_context["DisabledSetting"] = "On to Off" in operation_context["Change"]
            operation_context["EnabledSetting"] = "Off to On" in operation_context["Change"]

    return operation_context


def get_zoom_usergroup_context(event):
    """
    Parses the operation_detail field of Zoom.Operation events related to User Groups
    to provide usable fields for use in detections
    """
    operation_context = {}
    raw_string = event.get("operation_detail", "")
    category_type = event.get("category_type")
    action = event.get("action")

    if category_type == "User Group":
        if action == "Add":
            operation_context["GroupName"] = " ".join(raw_string.split()[2:])

        if action == "Delete":
            operation_context["GroupName"] = " ".join(raw_string.split()[1:])

        if action == "Update":
            columns = raw_string.split("-")
            operation_context["GroupName"] = " ".join(columns[0].split()[2:])
            operation_context["Change"] = columns[1].strip() if len(columns) > 1 else ""

            operation_context["DisabledSetting"] = "On to Off" in operation_context["Change"]
            operation_context["EnabledSetting"] = "Off to On" in operation_context["Change"]

    return operation_context


def get_zoom_room_context(event):
    """
    Parses the operation_detail field of Zoom.Operation events related to Zoom Meeting Rooms
    to provide usable fields for use in detections
    """
    operation_context = {}
    raw_string = event.get("operation_detail", "")
    category_type = event.get("category_type")
    action = event.get("action")

    if category_type == "Zoom Rooms":
        if action == "Update":
            operation_context["Parameter"] = raw_string.split("-")[0]
            operation_context["CurrentState"] = raw_string.split("-")[1].split(":")[1].strip()
            operation_context["PreviousState"] = raw_string.split("-")[2].split(":")[1].strip()
            operation_context["LockStatus"] = raw_string.split("-")[3].split(":")[1].strip()
            operation_context["Affected"] = raw_string.split("-")[4].split(":")[1].strip()

    return operation_context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_tailscale_helpers import is_tailscale_admin_console_event, tailscale_alert_context

tailscale_machine_approval_requirements_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="Machine Approval Requirements Disabled",
        expected_result=True,
        log={
            "event": {
                "action": "DISABLE",
                "actor": {
                    "displayName": "Homer Simpson",
                    "id": "uhfQwM4CNTRL",
                    "loginName": "homer.simpson@yourcompany.io",
                    "type": "USER",
                },
                "eventGroupID": "6dae044c7a8998599e94657b511d28f9",
                "origin": "ADMIN_CONSOLE",
                "target": {
                    "id": "panther.com",
                    "name": "panther.com",
                    "property": "MACHINE_APPROVAL_NEEDED",
                    "type": "TAILNET",
                },
            },
            "fields": {"recorded": "2023-06-27 22:58:15.824694387"},
            "p_any_emails": ["homer.simpson@yourcompany.io"],
            "p_any_md5_hashes": ["6dae044c7a8998599e94657b511d28f9"],
            "p_any_usernames": ["homersimpson"],
            "p_event_time": "2023-06-27 23:02:08.54",
            "p_log_type": "Custom.TailscaleAudit",
            "p_parse_time": "2023-06-27 23:02:08.54",
            "p_row_id": "8ec49771e630f6e8e5fdb28319d95c",
            "p_schema_version": 4,
            "p_source_id": "3acfbe1d-f7b2-4c3f-a26e-8da418fd29ef",
            "p_source_label": "Custom Tailscale Audit",
            "p_timeline": "2023-06-27 23:02:08.54",
            "p_udm": {},
            "time": 1687906694.915,
        },
    ),
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "event": {
                "action": "CREATE",
                "actor": {
                    "displayName": "Homer Simpson",
                    "id": "uodc9f3CNTRL",
                    "loginName": "homer.simpson@yourcompany.io",
                    "type": "USER",
                },
                "eventGroupID": "9f880e02981e341447958344b7b4071f",
                "new": {},
                "origin": "ADMIN_CONSOLE",
                "target": {"id": "k6r3fm3CNTRL", "name": "API key", "type": "API_KEY"},
            },
            "fields": {"recorded": "2023-07-19 16:11:41.778839718"},
            "p_any_actor_ids": ["uodc9f3CNTRL"],
            "p_any_emails": ["homer.simpson@yourcompany.io"],
            "p_any_usernames": ["homersimpson"],
            "p_event_time": "2023-07-19 16:11:41.601000",
            "p_log_type": "Tailscale.Audit",
            "p_parse_time": "2023-07-19 16:14:56.865276",
            "p_row_id": "02eaf97ec9caaaabff8882ba19ad1d",
            "p_schema_version": 0,
            "p_source_id": "5d65e24a-7ebb-403b-803c-51396e03d201",
            "p_source_label": "Tailscale Audit and Network Logs",
            "p_udm": {},
            "time": "2023-07-19 16:11:41.601000000",
        },
    ),
]


@panther_managed
class TailscaleMachineApprovalRequirementsDisabled(Rule):
    default_description = "A Tailscale User disabled machine approval requirement settings in your organization's tenant. This means devices can access your network without requiring approval."
    display_name = "Tailscale Machine Approval Requirements Disabled"
    default_runbook = "Assess if this was done by the user for a valid business reason. Be vigilant to re-enable this setting as it's in the best security interest for your organization's security posture."
    default_reference = "https://tailscale.com/kb/1099/device-approval/"
    default_severity = Severity.HIGH
    log_types = [LogType.TAILSCALE_AUDIT]
    id = "Tailscale.Machine.Approval.Requirements.Disabled-prototype"
    tests = tailscale_machine_approval_requirements_disabled_tests

    def rule(self, event):
        action = deep_get(event, "event", "action", default="<NO_ACTION_FOUND>")
        target_property = deep_get(event, "event", "target", "property", default="<NO_TARGET_PROPERTY_FOUND>")
        return all(
            [
                action == "DISABLE",
                target_property == "MACHINE_APPROVAL_NEEDED",
                is_tailscale_admin_console_event(event),
            ],
        )

    def title(self, event):
        user = deep_get(event, "event", "actor", "loginName", default="<NO_USER_FOUND>")
        return f"Tailscale user [{user}] disabled device approval requirements for new devices accessing your organization’s network."

    def alert_context(self, event):
        return tailscale_alert_context(event)



"""The default global accessible via the Panther web UI."""
# Define common code here that all of your policies and rules can share.
#
# Example usage:
#
# import panther_default
# def policy(resource):
#     return panther.example_helper()
#

import base64
import binascii
from typing import List


def example_helper():
    return True


AWS_ACCOUNTS = {
    # Add your AWS account IDs/names below:
    "123456789012": "sample-account",
}


def lookup_aws_account_name(account_id):
    """
    Lookup the AWS account name, return the ID if not found

    Args:
    ----
        account_id (str): The AWS account ID

    Returns:
    -------
        str: The name of the AWS account ID
        or
        str: The AWS account ID (unnamed account)

    """
    return AWS_ACCOUNTS.get(account_id, f"{account_id} (unnamed account)")


def aws_cloudtrail_success(event):
    if event.get("errorCode", "") or event.get("errorMessage", ""):
        return False
    return True


def aws_event_tense(event_name):
    """
    Convert an AWS CloudTrail eventName to be interpolated in alert titles

    An example is passing in StartInstance and returning 'started'.
    This would then be used in an alert title such as
    'The EC2 instance my-instance was started'.

    Args:
    ----
        event_name (str): The CloudTrail eventName

    Returns:
    -------
        str: A tensed version of the event name

    """
    mapping = {
        "Create": "created",
        "Delete": "deleted",
        "Start": "started",
        "Stop": "stopped",
        "Update": "updated",
    }
    for event_prefix, tensed in mapping.items():
        if event_name.startswith(event_prefix):
            return tensed
    # If the event pattern doesn't exist, return original
    return event_name


# Adapted from https://medium.com/@TalBeerySec/a-short-note-on-aws-key-id-f88cc4317489
def aws_key_account_id(aws_key: str):
    """Retrieve the AWS account ID associated with a given access key ID"""
    key_no_prefix = aws_key[4:]  # remove the four-character prefix
    base32_key = base64.b32decode(key_no_prefix)  # remainder of the key is base32-encoded
    decoded_key = base32_key[0:6]  # retrieve the 10-byte string

    # Convert the 10-byte string to an integer
    key_id_int = int.from_bytes(decoded_key, byteorder="big", signed=False)
    mask = int.from_bytes(binascii.unhexlify(b"7fffffffff80"), byteorder="big", signed=False)

    # Do a bitwise AND with the mask to retrieve the account ID
    # (divide the 10-byte key integer by 128 and remove the fractional part(s))
    account_id = (key_id_int & mask) >> 7
    return str(account_id)


def aws_regions() -> List[str]:
    """Return a list of AWS regions"""
    return [
        "ap-east-1",
        "ap-northeast-1",
        "ap-northeast-1",
        "ap-northeast-2",
        "ap-northeast-3",
        "ap-south-1",
        "ap-south-2",
        "ap-southeast-1",
        "ap-southeast-2",
        "ap-southeast-3",
        "ap-southeast-4",
        "ca-central-1",
        "ca-central-1",
        "eu-central-1",
        "eu-central-1",
        "eu-central-2",
        "eu-north-1",
        "eu-north-1",
        "eu-south-1",
        "eu-south-2",
        "eu-west-1",
        "eu-west-1",
        "eu-west-2",
        "eu-west-2",
        "eu-west-3",
        "eu-west-3",
        "il-central-1",
        "me-central-1",
        "me-south-1",
        "sa-east-1",
        "us-east-1",
        "us-east-2",
        "us-west-1",
        "us-west-2",
    ]



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType


def get_event_type(event):
    # currently, only tracking a few event types
    if event.get("event_type") == "FAILED_LOGIN":
        return event_type.FAILED_LOGIN
    if event.get("event_type") == "LOGIN":
        return event_type.SUCCESSFUL_LOGIN
    return None


class StandardBoxEvent(DataModel):
    id: str = "Standard.Box.Event"
    display_name: str = "Box Events"
    enabled: bool = True
    log_types: list[str] = [LogType.BOX_EVENT]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="$.created_by.name"),
        DataModelMapping(name="event_type", method=get_event_type),
        DataModelMapping(name="source_ip", path="ip_address"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk
from pypanther.helpers.panther_mongodb_helpers import mongodb_alert_context

mongo_db_atlas_api_key_created_tests: list[RuleTest] = [
    RuleTest(
        name="API Key Deleted",
        expected_result=False,
        log={
            "created": "2023-06-14 15:47:15",
            "currentvalue": {},
            "eventtypename": "API_KEY_ACCESS_LIST_ENTRY_DELETED",
            "id": "1234abcd13f2804962409423",
            "isglobaladmin": False,
            "links": [
                {
                    "href": "https://cloud.mongodb.com/api/atlas/v1.0/orgs/9876xyz123lmnop0/events/1234abcd13f2804962409423",
                    "rel": "self",
                },
            ],
            "orgid": "9876xyz123lmnop0",
            "p_event_time": "2023-06-14 15:47:15",
            "p_log_type": "MongoDB.OrganizationEvent",
            "p_parse_time": "2023-06-14 15:53:42.415",
            "p_row_id": "5ad9c2df49e19aac98def9e118e236",
            "p_schema_version": 0,
            "p_source_id": "a2e8f928-c6e5-4110-b6f9-1b741176041d",
            "p_source_label": "mongo-test-2",
            "remoteaddress": "1.2.3.4",
            "targetpublickey": "xfvcfwtt",
            "userid": "abcd1234userid988",
            "username": "user@company.com",
            "whitelistentry": "1.2.3.4",
        },
    ),
    RuleTest(
        name="API Key Created",
        expected_result=True,
        log={
            "created": "2023-06-14 15:47:15",
            "currentvalue": {},
            "eventtypename": "API_KEY_ACCESS_LIST_ENTRY_ADDED",
            "id": "1234abcd13f2804962409423",
            "isglobaladmin": False,
            "links": [
                {
                    "href": "https://cloud.mongodb.com/api/atlas/v1.0/orgs/9876xyz123lmnop0/events/1234abcd13f2804962409423",
                    "rel": "self",
                },
            ],
            "orgid": "9876xyz123lmnop0",
            "p_event_time": "2023-06-14 15:47:15",
            "p_log_type": "MongoDB.OrganizationEvent",
            "p_parse_time": "2023-06-14 15:53:42.415",
            "p_row_id": "5ad9c2df49e19aac98def9e118e236",
            "p_schema_version": 0,
            "p_source_id": "a2e8f928-c6e5-4110-b6f9-1b741176041d",
            "p_source_label": "mongo-test-2",
            "remoteaddress": "1.2.3.4",
            "targetpublickey": "xfvcfwtt",
            "userid": "abcd1234userid988",
            "username": "user@company.com",
            "whitelistentry": "1.2.3.4",
        },
    ),
]


@panther_managed
class MongoDBAtlasApiKeyCreated(Rule):
    default_description = "A MongoDB Atlas api key's access list was updated"
    display_name = "MongoDB Atlas API Key Created"
    default_severity = Severity.MEDIUM
    default_reference = "https://www.mongodb.com/docs/atlas/configure-api-access/#std-label-about-org-api-keys"
    log_types = [LogType.MONGODB_ORGANIZATION_EVENT]
    id = "MongoDB.Atlas.ApiKeyCreated-prototype"
    tests = mongo_db_atlas_api_key_created_tests

    def rule(self, event):
        return deep_get(event, "eventTypeName", default="") == "API_KEY_ACCESS_LIST_ENTRY_ADDED"

    def title(self, event):
        user = deep_get(event, "username", default="<USER_NOT_FOUND>")
        public_key = deep_get(event, "targetPublicKey", default="<PUBLIC_KEY_NOT_FOUND>")
        return f"MongoDB Atlas: [{user}] updated the allowed access list for API Key [{public_key}]"

    def alert_context(self, event):
        context = mongodb_alert_context(event)
        links = deep_walk(event, "links", "href", return_val="first", default="<LINKS_NOT_FOUND>")
        extra_context = {
            "links": links,
            "event_type_name": deep_get(event, "eventTypeName", default="<EVENT_TYPE_NOT_FOUND>"),
            "target_public_key": deep_get(event, "targetPublicKey", default="<PUBLIC_KEY_NOT_FOUND>"),
        }
        context.update(extra_context)
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import ZENDESK_CHANGE_DESCRIPTION

zendesk_user_suspension_tests: list[RuleTest] = [
    RuleTest(
        name="Zendesk - Suspension Enabled",
        expected_result=True,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "user_setting",
            "source_label": "Suspension state: Bob Cat",
            "action": "create",
            "change_description": "Suspended",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Zendesk - Suspension Disabled",
        expected_result=True,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "user_setting",
            "source_label": "Suspension state: Bob Cat",
            "action": "update",
            "change_description": "Unsuspended",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Zendesk - Admin Role Assigned",
        expected_result=False,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "user",
            "source_label": "Bob Cat",
            "action": "update",
            "change_description": "Role changed from End User to Administrator",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
]


@panther_managed
class ZendeskUserSuspension(Rule):
    id = "Zendesk.UserSuspension-prototype"
    display_name = "Zendesk User Suspension Status Changed"
    log_types = [LogType.ZENDESK_AUDIT]
    tags = ["Zendesk", "Impact:Account Access Removal"]
    reports = {"MITRE ATT&CK": ["TA0040:T1531"]}
    default_severity = Severity.HIGH
    default_description = "A user's Zendesk suspension status was changed."
    default_runbook = "Ensure the user's suspension status is appropriate."
    default_reference = "https://support.zendesk.com/hc/en-us/articles/4408889293978-Suspending-a-user#:~:text=select%20Unsuspend%20access.-,Identifying%20suspended%20users,name%20on%20the%20Customers%20page"
    summary_attributes = ["p_any_ip_addresses"]
    tests = zendesk_user_suspension_tests
    USER_SUSPENSION_ACTIONS = {"create", "update"}

    def rule(self, event):
        return (
            event.get("source_type") == "user_setting"
            and event.get("action", "") in self.USER_SUSPENSION_ACTIONS
            and ("suspended" in event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower())
        )

    def title(self, event):
        suspension_status = event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower()
        user = event.get("source_label", "<UNKNOWN_USER>").split(":")
        if len(user) > 1:
            user = user[1].strip()
        return f"Actor user [{event.udm('actor_user')}] {suspension_status} user [{user}]"

    def severity(self, event):
        if event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower() == "suspended":
            return "INFO"
        return "HIGH"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_login_type_tests: list[RuleTest] = [
    RuleTest(
        name="Login With Approved Type",
        expected_result=False,
        log={
            "id": {"applicationName": "login"},
            "actor": {"email": "some.user@somedomain.com"},
            "type": "login",
            "name": "login_success",
            "parameters": {"login_type": "saml"},
        },
    ),
    RuleTest(
        name="Login With Unapproved Type",
        expected_result=True,
        log={
            "id": {"applicationName": "login"},
            "actor": {"email": "some.user@somedomain.com"},
            "type": "login",
            "name": "login_success",
            "parameters": {"login_type": "turbo-snail"},
        },
    ),
    RuleTest(
        name="Non-Login event",
        expected_result=False,
        log={
            "id": {"applicationName": "logout"},
            "actor": {"email": "some.user@somedomain.com"},
            "type": "login",
            "name": "login_success",
            "parameters": {"login_type": "saml"},
        },
    ),
    RuleTest(
        name="Saml Login Event",
        expected_result=False,
        log={
            "actor": {"email": "some.user@somedomain.com"},
            "id": {"applicationName": "saml", "time": "2022-05-26 15:26:09.421000000"},
            "ipAddress": "10.10.10.10",
            "kind": "admin#reports#activity",
            "name": "login_success",
            "parameters": {
                "application_name": "Some SAML Application",
                "initiated_by": "sp",
                "orgunit_path": "/SomeOrgUnit",
                "saml_status_code": "SUCCESS_URI",
            },
            "type": "login",
        },
    ),
]


@panther_managed
class GSuiteLoginType(Rule):
    id = "GSuite.LoginType-prototype"
    display_name = "GSuite Login Type"
    enabled = False
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite", "Configuration Required", "Initial Access:Valid Accounts"]
    reports = {"MITRE ATT&CK": ["TA0001:T1078"]}
    default_severity = Severity.MEDIUM
    default_description = "A login of a non-approved type was detected for this user.\n"
    default_reference = "https://support.google.com/a/answer/9039184?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Correct the user account settings so that only logins of approved types are available.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_login_type_tests
    # allow-list of approved login types
    APPROVED_LOGIN_TYPES = {"exchange", "google_password", "reauth", "saml", "unknown"}
    # allow-list any application names here
    APPROVED_APPLICATION_NAMES = {"saml"}

    def rule(self, event):
        if event.get("type") != "login":
            return False
        if event.get("name") == "logout":
            return False
        if (
            deep_get(event, "parameters", "login_type") in self.APPROVED_LOGIN_TYPES
            or deep_get(event, "id", "applicationName") in self.APPROVED_APPLICATION_NAMES
        ):
            return False
        return True

    def title(self, event):
        return f"A login attempt of a non-approved type was detected for user [{deep_get(event, 'actor', 'email', default='<UNKNOWN_USER>')}]"



from ipaddress import ip_network

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context

awsvpc_inbound_port_blacklist_tests: list[RuleTest] = [
    RuleTest(
        name="Public to Private IP on Restricted Port",
        expected_result=True,
        log={"dstPort": 22, "dstAddr": "10.0.0.1", "srcAddr": "1.1.1.1", "p_log_type": "AWS.VPCFlow"},
    ),
    RuleTest(
        name="Public to Private IP on Allowed Port",
        expected_result=False,
        log={"dstPort": 443, "dstAddr": "10.0.0.1", "srcAddr": "1.1.1.1", "p_log_type": "AWS.VPCFlow"},
    ),
    RuleTest(
        name="Private to Private IP on Restricted Port",
        expected_result=False,
        log={"dstPort": 22, "dstAddr": "10.0.0.1", "srcAddr": "10.10.10.1", "p_log_type": "AWS.VPCFlow"},
    ),
    RuleTest(
        name="Public to Private IP on Restricted Port - OCSF",
        expected_result=True,
        log={
            "dst_endpoint": {"ip": "10.0.0.1", "port": 22},
            "src_endpoint": {"ip": "1.1.1.1"},
            "p_log_type": "OCSF.NetworkActivity",
        },
    ),
    RuleTest(
        name="Public to Private IP on Allowed Port - OCSF",
        expected_result=False,
        log={
            "dst_endpoint": {"ip": "10.0.0.1", "port": 443},
            "src_endpoint": {"ip": "1.1.1.1"},
            "p_log_type": "OCSF.NetworkActivity",
        },
    ),
]


@panther_managed
class AWSVPCInboundPortBlacklist(Rule):
    id = "AWS.VPC.InboundPortBlacklist-prototype"
    display_name = "VPC Flow Logs Inbound Port Blocklist"
    enabled = False
    log_types = [LogType.AWS_VPC_FLOW, LogType.OCSF_NETWORK_ACTIVITY]
    tags = ["AWS", "DataModel", "Configuration Required", "Security Control", "Command and Control:Non-Standard Port"]
    reports = {"MITRE ATT&CK": ["TA0011:T1571"]}
    default_reference = "https://docs.aws.amazon.com/vpc/latest/userguide/flow-logs.html"
    default_severity = Severity.HIGH
    default_description = "VPC Flow Logs observed inbound traffic violating the port blocklist.\n"
    default_runbook = "Block the unapproved traffic, or update the unapproved ports list.\n"
    summary_attributes = ["srcaddr", "dstaddr", "dstport"]
    tests = awsvpc_inbound_port_blacklist_tests
    CONTROLLED_PORTS = {22, 3389}

    def rule(self, event):
        # Only monitor for blocklisted ports
        #
        # Defaults to True (no alert) if 'dstport' is not present
        if event.udm("destination_port") not in self.CONTROLLED_PORTS:
            return False
        # Only monitor for traffic coming from non-private IP space
        #
        # Defaults to True (no alert) if 'srcaddr' key is not present
        source_ip = event.udm("source_ip") or "0.0.0.0/32"
        if not ip_network(source_ip).is_global:
            return False
        # Alert if the traffic is destined for internal IP addresses
        #
        # Defaults to False(no alert) if 'dstaddr' key is not present
        destination_ip = event.udm("destination_ip") or "1.0.0.0/32"
        return not ip_network(destination_ip).is_global

    def alert_context(self, event):
        return aws_rule_context(event)
